import asyncio
import logging
import time
from http import HTTPStatus
from typing import AsyncGenerator, Dict, List, Tuple

import aiohttp
import json

from evalscope.perf.arguments import Arguments
from evalscope.perf.utils.local_server import ServerSentEvent
from evalscope.utils.logger import get_logger

logger = get_logger()


class AioHttpClient:

    def __init__(
        self,
        args: Arguments,
    ):
        self.url = args.url
        self.debug = args.debug
        self.headers = {'user-agent': 'modelscope_bench', **(args.headers or {})}
        self.read_timeout = args.read_timeout
        self.connect_timeout = args.connect_timeout
        self.client = aiohttp.ClientSession(
            timeout=aiohttp.ClientTimeout(
                total=self.read_timeout + self.connect_timeout,
                connect=self.connect_timeout,
                sock_read=self.read_timeout),
            connector=aiohttp.TCPConnector(limit=1),
            trace_configs=[self._create_trace_config()] if self.debug else [])
        if self.debug:
            get_logger(log_level=logging.DEBUG)

    def _create_trace_config(self):
        trace_config = aiohttp.TraceConfig()
        trace_config.on_request_start.append(self.on_request_start)
        trace_config.on_request_chunk_sent.append(self.on_request_chunk_sent)
        trace_config.on_response_chunk_received.append(self.on_response_chunk_received)
        return trace_config

    async def __aenter__(self):
        pass

    async def __aexit__(self, exc_type, exc, tb):
        await self.client.close()

    async def _handle_stream(self, response: aiohttp.ClientResponse):
        is_error = False
        async for line in response.content:
            line = line.decode('utf8').rstrip('\n\r')
            sse_msg = ServerSentEvent.decode(line)
            if sse_msg:
                logger.debug(f'Response recevied: {line}')
                if sse_msg.event == 'error':
                    is_error = True
                if sse_msg.data:
                    if sse_msg.data.startswith('[DONE]'):
                        break
                    yield is_error, response.status, sse_msg.data

    async def _handle_response(self, response: aiohttp.ClientResponse) -> AsyncGenerator[Tuple[bool, int, str], None]:
        response_status = response.status
        response_content_type = response.content_type
        content_type_json = 'application/json'
        content_type_event_stream = 'text/event-stream'
        is_success = response_status == HTTPStatus.OK

        if is_success:
            # Handle successful response with 'text/event-stream' content type
            if content_type_event_stream in response_content_type:
                async for is_error, response_status, content in self._handle_stream(response):
                    yield (is_error, response_status, content)
            # Handle successful response with 'application/json' content type
            elif content_type_json in response_content_type:
                content = await response.json()
                if content.get('object') == 'error':
                    yield (True, content.get('code'), content.get('message'))  # DashScope
                else:
                    yield (False, response_status, json.dumps(content, ensure_ascii=False))
            # Handle other successful responses
            else:
                content = await response.read()
                yield (False, response_status, content)
        else:
            # Handle error response with 'application/json' content type
            if content_type_json in response_content_type:
                error = await response.json()
                yield (True, response_status, json.dumps(error, ensure_ascii=False))
            # Handle error response with 'text/event-stream' content type
            elif content_type_event_stream in response_content_type:
                async for _, _, data in self._handle_stream(response):
                    error = json.loads(data)
                    yield (True, response_status, json.dumps(error, ensure_ascii=False))
            # Handle other error responses
            else:
                msg = await response.read()
                yield (True, response_status, msg.decode('utf-8'))

    async def post(self, body):
        headers = {'Content-Type': 'application/json', **self.headers}
        try:
            data = json.dumps(body, ensure_ascii=False)  # serialize to JSON
            async with self.client.request('POST', url=self.url, data=data, headers=headers) as response:
                async for rsp in self._handle_response(response):
                    yield rsp
        except (aiohttp.ClientConnectorError, Exception) as e:
            logger.error(e)
            yield (True, None, e)

    @staticmethod
    async def on_request_start(session, context, params: aiohttp.TraceRequestStartParams):
        logger.debug(f'Starting request: <{params}>')

    @staticmethod
    async def on_request_chunk_sent(session, context, params: aiohttp.TraceRequestChunkSentParams):
        method = params.method
        url = params.url
        chunk = params.chunk.decode('utf-8')
        max_length = 100
        if len(chunk) > 2 * max_length:
            truncated_chunk = f'{chunk[:max_length]}...{chunk[-max_length:]}'
        else:
            truncated_chunk = chunk
        logger.debug(f'Request sent: <{method=},  {url=}, {truncated_chunk=}>')

    @staticmethod
    async def on_response_chunk_received(session, context, params: aiohttp.TraceResponseChunkReceivedParams):
        method = params.method
        url = params.url
        chunk = params.chunk.decode('utf-8')
        max_length = 200
        if len(chunk) > 2 * max_length:
            truncated_chunk = f'{chunk[:max_length]}...{chunk[-max_length:]}'
        else:
            truncated_chunk = chunk
        logger.debug(f'Request received: <{method=},  {url=}, {truncated_chunk=}>')


async def test_connection(args: Arguments) -> bool:
    is_error = True
    start_time = time.perf_counter()

    async def attempt_connection():
        client = AioHttpClient(args)
        async with client:
            if 'chat/completions' in args.url:
                request = {'messages': [{'role': 'user', 'content': 'hello'}], 'model': args.model}
            else:
                request = {'prompt': 'hello', 'model': args.model}
            async for is_error, state_code, response_data in client.post(request):
                return is_error, state_code, response_data

    while True:
        try:
            is_error, state_code, response_data = await asyncio.wait_for(
                attempt_connection(), timeout=args.connect_timeout)
            if not is_error:
                logger.info('Connection successful.')
                return True
            logger.warning(f'Retrying...  <{state_code}> {response_data}')
        except Exception as e:
            logger.warning(f'Retrying... <{e}>')

        if time.perf_counter() - start_time >= args.connect_timeout:
            logger.error('Overall connection attempt timed out.')
            return False

        await asyncio.sleep(10)



# Copyright (c) Alibaba, Inc. and its affiliates.
# Adopted from tatsu-lab@stanford_alpaca. Below is the original copyright:
#    Copyright 2023 Rohan Taori, Ishaan Gulrajani, Tianyi Zhang, Yann Dubois, Xuechen Li
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

# Make it more memory efficient by monkey patching the LLaMA model with FlashAttn.

from dataclasses import dataclass, field
import json
import os
from rouge import Rouge
import time
from urllib3.exceptions import MaxRetryError, NewConnectionError
import requests


def evaluate_rouge_l(cand_list: list, ref_list: list):
    if len(ref_list) == 0:
        return 0
    rouge = Rouge()
    rouge_score = rouge.get_scores(hyps=cand_list, refs=ref_list, avg=True)
    rougel = rouge_score["rouge-l"]["f"]
    return rougel


def nested_load_test_data(data_path):
    test_raw_data = []
    if os.path.isdir(data_path):
        for f in os.listdir(data_path):
            temp_test = nested_load_test_data(os.path.join(data_path, f))
            test_raw_data += temp_test
        return test_raw_data
    elif os.path.isfile(data_path) and data_path.endswith('.json'):
        print("Load data from", data_path)
        temp_data = json.load(open(data_path, "r"))
        test_raw_data = temp_data
        return test_raw_data
    else:
        return []


def baichuan_call(context: list, system: str):
    url = "https://api.baichuan-ai.com/v1/chat/completions"
    api_key = "sk-xxx"

    new_msg = []
    new_msg.append({
        "role": 'system',
        'content': system})
    for m in context:
        if m['role'] == "user":
            new_msg.append({
                'role': 'user', 'content': m['content']
            })
        elif m['role'] == "function":
            new_msg.append({
                'role': 'user', 'content': m['content']
            })
        elif m['role'] == 'assistant':
            new_msg.append({
                'role': 'assistant', 'content': m['content']
            })
    # print(json.dumps(new_msg, indent=2))
    data = {
        "model": "Baichuan2-Turbo",
        "messages": new_msg,
        "stream": False
    }

    json_data = json.dumps(data)

    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + api_key
    }

    for i in range(5):
        res = None
        try:
            res = requests.post(url, data=json_data, headers=headers, timeout=60)
            res = res._content.decode('utf-8')
            res = json.loads(res)
            return res["choices"][0]["message"]["content"]
        except KeyError:
            print(res)
            time.sleep(1)
            continue
        except ConnectionError:
            time.sleep(5)
            continue
        except MaxRetryError:
            time.sleep(5)
            continue
        except NewConnectionError:
            time.sleep(5)
            continue
    return ""


def minimax_call(context: list, system: str):
    group_id = "your-id"
    api_key = "your-xxx"

    url = f"https://api.minimax.chat/v1/text/chatcompletion_pro?GroupId={group_id}"
    headers = {"Authorization": f"Bearer {api_key}", "Content-Type": "application/json"}

    # construct message
    system_prompt = "MM智能助理是一款由MiniMax自研的，没有调用其他产品的接口的大型语言模型。" \
                    "MiniMax是一家中国科技公司，一直致力于进行大模型相关的研究。"
    system_prompt += ('\n' + system)

    new_msg = []
    for m in context:
        if m['role'] == "user":
            new_msg.append({
                'sender_type': 'USER', 'sender_name': 'user', 'text': m['content']
            })
        elif m['role'] == "function":
            new_msg.append({
                'sender_type': 'USER', 'sender_name': 'funtion', 'text': m['content']
            })
        elif m['role'] == 'assistant':
            new_msg.append({
                'sender_type': 'BOT', 'sender_name': 'MM智能助理', 'text': m['content']
            })

    request_body = {
        "model": "abab6-chat",
        # "model": "abab5.5s-chat",
        "tokens_to_generate": 8192,
        "reply_constraints": {"sender_type": "BOT", "sender_name": "MM智能助理"},
        "messages": new_msg,
        "bot_setting": [
            {
                "bot_name": "MM智能助理",
                "content": system_prompt,
            }
        ],
    }
    response = requests.post(url, headers=headers, json=request_body)
    status_code = response.status_code
    for i in range(5):
        try:
            if status_code == 200:
                reply = response.json()["reply"]
                if len(reply) == 0:
                    print("limit rate")
                    time.sleep(8)
                    continue
                print(f'>>return: {reply}')
                return reply
            else:
                print(response._content)
                time.sleep(5)
        except KeyError:
            print(response)
            time.sleep(5)
            continue
    return ""


def swift_call(context: list, system: str, swift_infer_obj):
    query_d: dict = context[-1]
    history_list = context[: -1]

    query: str = query_d['content']
    history_msg = []

    tmp_list = []
    for idx, item in enumerate(history_list):

        if idx % 2 == 0:
            tmp_list.append(item['content'])
        else:
            tmp_list.append(item['content'])
            history_msg.append(tuple(tmp_list))
            tmp_list = []

    try:
        resp_str: str = swift_infer_obj.predict(system=system, query=query, history=history_msg)
    except Exception as e:
        print(e)
        resp_str = ''

    return resp_str


@dataclass
class InferArgs:
    model_name_or_path: str
    model_type: str
    data_path: str
    output_dir: str
    deploy_type: str
    max_new_tokens: int = 2048
    num_infer_samples: int = None


def run_infer(args: InferArgs):

    if args.deploy_type == 'swift':
        from evalscope.third_party.toolbench_static.llm.swift_infer import SwiftInfer, SwiftInferArgs
        swift_infer_args = SwiftInferArgs(model_id_or_path=args.model_name_or_path,
                                          model_type=args.model_type,
                                          max_new_tokens=args.max_new_tokens)
        swift_infer = SwiftInfer(args=swift_infer_args)
    else:
        swift_infer = None

    # load data
    infer_samples = nested_load_test_data(args.data_path)
    if args.num_infer_samples is not None:
        infer_samples = infer_samples[:args.num_infer_samples]

    os.makedirs(args.output_dir, exist_ok=True)
    if os.path.exists(os.path.join(args.output_dir, 'predictions.json')):
        with open(os.path.join(args.output_dir, 'predictions.json')) as f:
            processed_samples = json.load(f)
    else:
        processed_samples = []
    preds = []
    refs = []
    for i, o in enumerate(infer_samples):
        if i < len(processed_samples) and "predictions" in processed_samples[i].keys():
            infer_samples[i]['predictions'] = processed_samples[i]['predictions']
            refs.append(processed_samples[i]['target'])
            preds.append(processed_samples[i]['predictions'])
            continue

        system = o['messages'][0]['content']
        new_msg = o['messages'][1:]

        print('================================')
        print('case', str(i))

        if args.deploy_type == 'minimax':
            response_text = minimax_call(new_msg, system)
        # elif model_args.model_type == 'xingchen':
        #     response_text = spark_call(new_msg, system)
        # elif model_args.model_type == 'xingchen_v2':
        #     response_text = spark_call_v2(new_msg, system, model_args)
        elif args.deploy_type == 'baichuan':
            response_text = baichuan_call(new_msg, system)
        elif args.deploy_type == 'swift':
            assert swift_infer is not None, 'ModelScope Swift infer process is not initialized.'
            response_text = swift_call(new_msg, system, swift_infer)
        else:
            raise NotImplementedError

        candidate = response_text
        print(candidate)
        if candidate.startswith(': '):
            candidate = candidate[2:]
        if candidate.strip() in ['', '.', ',']:
            candidate = 'none'
        reference = infer_samples[i]['target']
        infer_samples[i]['predictions'] = candidate
        if reference.strip() in ['', '.', ',']:
            reference = "none"
        refs.append(reference)
        preds.append(candidate)

        with open(os.path.join(args.output_dir, 'predictions.json'), 'w') as f:
            json.dump(infer_samples[:i + 1], f, indent=4)

    rouge_l = round(evaluate_rouge_l(preds, refs), 2)
    print('\n*** Overall rouge:', rouge_l)



# Copyright (c) Alibaba, Inc. and its affiliates.
import os.path
from typing import List
import re
import json

from evalscope.third_party.longbench_write.eval import EvalLength
from evalscope.third_party.longbench_write.utils import count_words, chinese_to_arabic
from evalscope.utils import jsonl_to_list
from evalscope.utils.logger import get_logger

logger = get_logger()

"""
This script is used to preprocess the dataset for the LongWriter.
"""


class DataETL:

    def __init__(self, in_file: str, out_dir: str):
        self.data: List[dict] = jsonl_to_list(in_file)
        logger.info(f'Loaded {len(self.data)} samples from {in_file}')

        self.out_dir = out_dir

    @staticmethod
    def filter(strategies: List[str], example: dict, verbose: bool = False) -> dict:
        # example = {'messages': [{'role': 'user', 'content': 'xxx'}, {'role': 'assistant', 'content': 'xxx'}]}

        for strategy in strategies:
            if strategy == 'no_required_length':
                pattern1 = r'(\d+)字'
                pattern2 = r'(.)百字|(.)千字|(.)万字'
                pattern3 = r'(\d+) words'

                text = example['messages'][0]['content']
                matches = re.findall(pattern1, text)
                matches.extend(re.findall(pattern2, text))
                matches.extend(re.findall(pattern3, text))
                matches = list(set(matches))

                final_matches = []
                for item in matches:
                    if isinstance(item, tuple):
                        # Note: refer to pattern2
                        for idx in range(len(item)):
                            num = chinese_to_arabic(item[idx])
                            if idx == 0 and item[idx]:
                                if isinstance(num, int):
                                    final_matches.append(num * 100)
                            elif idx == 1 and item[idx]:
                                if isinstance(num, int):
                                    final_matches.append(num * 1000)
                            elif idx == 2 and item[idx]:
                                if isinstance(num, int):
                                    final_matches.append(num * 10000)
                            else:
                                continue
                    else:
                        final_matches.append(item)

                final_matches = list(set(final_matches))
                if verbose and len(final_matches) > 0:
                    logger.info(f'>>final_matches: {final_matches}')

                if len(final_matches) == 1:
                    required_length = int(final_matches[0])
                    example['length'] = required_length
                else:
                    example = {}
            else:
                raise ValueError(f'Unknown strategy: {strategy}')

        return example

    def process_filter(self):
        # Entry point for processing the data
        filtered_data = []
        for ex in self.data:
            filtered_ex = DataETL.filter(strategies=['no_required_length'], example=ex, verbose=True)
            if len(filtered_ex) > 0:
                filtered_data.append(filtered_ex)

        for example in filtered_data:
            assistant_messages = example['messages'][1]
            assert assistant_messages['role'] == 'assistant'
            response_length, _ = count_words(assistant_messages['content'])
            example['response_length'] = response_length

        # Dump the filtered data to the output file
        # example = {'messages': [{'role': 'user', 'content': 'xxx'}, {'role': 'assistant', 'content': 'xxx'}], 'length': 1000, 'response_length': 1000'}
        out_file = os.path.join(self.out_dir, 'long_filtered.jsonl')
        with open(out_file, 'w', encoding='utf-8') as f:
            for example in filtered_data:
                f.write(json.dumps(example, ensure_ascii=False) + '\n')

        return out_file

    def process_eval_length(self, in_file: str, threshold: float = 80):
        data_list = jsonl_to_list(in_file)

        res_list = []
        x_list = []
        y_list = []
        x_filter_list = []
        y_filter_list = []
        for ex in data_list:
            x = ex['length']
            y = ex['response_length']
            if x == 0 or y == 0:
                continue

            if x / y > 3:
                print(f'\n>>ex: {json.dumps(ex, ensure_ascii=False)}')
                print(f'>>length: {x}, response_length: {y}\n')

            x_list.append(x)
            y_list.append(y)

            len_score = EvalLength.score(x, y)
            if len_score >= threshold:
                ex['score_length'] = len_score
                res_list.append(ex)

                x_filter_list.append(x)
                y_filter_list.append(y)

        out_file = os.path.join(self.out_dir, 'long_filtered_length.jsonl')
        logger.info(f'Got {len(res_list)} examples left')
        with open(out_file, 'w', encoding='utf-8') as f:
            for example in res_list:
                f.write(json.dumps(example, ensure_ascii=False) + '\n')

        plt = EvalLength.plot_img(x_list, y_list)
        plt.savefig(os.path.join(self.out_dir, 'length_scatter.png'))

        plt_filter = EvalLength.plot_img(x_filter_list, y_filter_list)
        plt_filter.savefig(os.path.join(self.out_dir, 'length_scatter_filtered.png'))

        return out_file


if __name__ == "__main__":
    # run `no_required_length`: got 1748 exampels left

    # Refer to: https://modelscope.cn/datasets/ZhipuAI/LongWriter-6k/files
    longbench_write_file = '/path/to/long.jsonl'
    out_dir = '/path/to/your_output_dir'

    data_etl = DataETL(in_file=longbench_write_file, out_dir=out_dir)
    filtered_file_path = data_etl.process_filter()
    print(f'Filtered data is saved in {filtered_file_path}')

    eval_l_file_path = data_etl.process_eval_length(in_file=filtered_file_path, threshold=80)



# Copyright (c) Alibaba, Inc. and its affiliates.

import json
import threading
import time
from asyncio import Queue

import requests
from typing import Union, List, Optional, Dict
from concurrent.futures import ThreadPoolExecutor
from modelscope.utils.logger import get_logger

logger = get_logger()


class OpenaiApi:

    def __init__(self,
                 model: str,
                 openai_api_key,
                 openai_api_base,
                 logprobs: Optional[bool] = False,
                 top_logprobs: Optional[int] = None,
                 max_new_tokens: int = 4096,
                 temperature: Optional[float] = 0.0,
                 repetition_penalty: Optional[float] = 1.0,
                 is_chat: bool = True,
                 verbose: bool = True,
                 retry: int = 3,
                 query_per_second: int = 10,     # TODO
                 **kwargs):

        self.temperature = temperature
        self.repetition_penalty = repetition_penalty
        self.max_tokens = max_new_tokens
        self.logprobs = logprobs
        self.top_logprobs = top_logprobs

        self.openai_api_key = openai_api_key
        self.url = openai_api_base
        self.model = model
        self.is_chat = is_chat
        self.retry = retry
        self.verbose = verbose

        self.token_bucket = TokenBucket(query_per_second, verbose)

    def generate_simple(self, inputs: Union[List[str]]):

        def process_one(in_data: str):

            if self.is_chat:
                data = dict(
                    model=self.model,
                    messages=[{'role': 'user', 'content': in_data}],
                    max_tokens=self.max_tokens,
                    n=1,
                    logprobs=self.logprobs,
                    top_logprobs=self.top_logprobs,
                    stop=None,
                    temperature=self.temperature,
                    repetition_penalty=self.repetition_penalty,
                )
            else:
                data = dict(
                    model=self.model,
                    prompt=in_data,
                    max_tokens=self.max_tokens,
                    temperature=self.temperature,
                    repetition_penalty=self.repetition_penalty,
                )

            # todo
            openai_api_key = self.openai_api_key or ''
            header = {'Authorization': f'Bearer ', 'content-type': 'application/json', }
            data = json.dumps(data, ensure_ascii=False)

            if self.verbose:
                logger.info(f'>>data in generate_simple: {data}')

            resp = requests.post(self.url, headers=header, data=data)
            resp = resp.json()
            if self.verbose:
                logger.info(f'>>resp in generate_simple: {resp}')

            if self.logprobs:
                return resp['choices']
            else:
                if self.is_chat:
                    return resp['choices'][0]['message']['content'].strip()
                else:
                    return resp['choices'][0]['text'].strip()

        with ThreadPoolExecutor() as executor:
            results = list(executor.map(process_one, inputs))

        return results

    def generate(self,
                 inputs: Union[List[str], List[List]],
                 **kwargs) -> List[str]:
        """
        Generate responses from OpenAI API.

        Args:
            inputs: The input messages for the model. It can be a string or a list of messages.
                e.g. ['who are you ?', 'what is your name ?']
                e.g. [[{'role': 'user', 'content': 'who are you ?'}], ...]
            kwargs: The optional arguments for the model.
        """
        results = []
        # with ThreadPoolExecutor() as executor:
        #     results = list(executor.map(self._generate, inputs))

        for input in inputs:
            results.append(self._generate(input))

        return results

    def _generate(self, messages: Union[str, List[Dict]]) -> str:

        if isinstance(messages, str):
            messages = [{'role': 'user', 'content': messages}]

        max_num_retries = 0
        while max_num_retries < self.retry:
            # self.wait()

            header = {
                'Authorization': f'Bearer {self.openai_api_key}',
                'content-type': 'application/json',
            }

            try:
                if self.is_chat:
                    data = dict(
                        model=self.model,
                        messages=messages,
                        max_tokens=self.max_tokens,
                        n=1,
                        logprobs=self.logprobs,
                        top_logprobs=self.top_logprobs,
                        stop=None,
                        temperature=self.temperature,
                        repetition_penalty=self.repetition_penalty,
                    )
                else:
                    # TODO: This is a temporary solution for non-chat models.
                    input_prompts = []
                    for msg in messages:
                        input_prompts.append(msg['content'])

                    data = dict(
                        model=self.model,
                        prompt='\n'.join(input_prompts),
                        max_tokens=self.max_tokens,
                        temperature=self.temperature,
                        repetition_penalty=self.repetition_penalty,
                    )

                def remove_none_val(input_d: dict):
                    return {k: v for k, v in input_d.items() if v is not None}
                data = remove_none_val(data)

                if self.verbose:
                    logger.info(f'>> Post data: {json.dumps(data, ensure_ascii=False)}')
                raw_response = requests.post(self.url,
                                             headers=header,
                                             data=json.dumps(data, ensure_ascii=False))

                response = raw_response.json()
                if self.verbose:
                    logger.info(f'>> response: {response}')

                if self.logprobs:
                    return response['choices']
                else:
                    if self.is_chat:
                        return response['choices'][0]['message']['content'].strip()
                    else:
                        return response['choices'][0]['text'].strip()

            except Exception as e:
                logger.error(f'Error occurs: {str(e)}')
                max_num_retries += 1
                continue

    def wait(self):
        return self.token_bucket.get_token()


class TokenBucket:
    """A token bucket for rate limiting.

    Args:
        query_per_second (float): The rate of the token bucket.
    """

    def __init__(self, rate, verbose=False):
        self._rate = rate
        self._tokens = threading.Semaphore(0)
        self.started = False
        self._request_queue = Queue()
        self.logger = get_logger()
        self.verbose = verbose

    def _add_tokens(self):
        """Add tokens to the bucket."""
        while True:
            if self._tokens._value < self._rate:
                self._tokens.release()
            time.sleep(1 / self._rate)

    def get_token(self):
        """Get a token from the bucket."""
        if not self.started:
            self.started = True
            threading.Thread(target=self._add_tokens, daemon=True).start()
        self._tokens.acquire()
        if self.verbose:
            cur_time = time.time()
            while not self._request_queue.empty():
                if cur_time - self._request_queue.queue[0] > 60:
                    self._request_queue.get()
                else:
                    break
            self._request_queue.put(cur_time)
            self.logger.info(f'Current RPM {self._request_queue.qsize()}.')



# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright (c) OpenCompass.

import functools
import hashlib
import importlib
import importlib.util
import os
import random
import re
import sys
from typing import Any, Dict, List, Tuple, Union

import json
import jsonlines as jsonl
import numpy as np
import torch
import torch.nn.functional as F
import yaml

from evalscope.constants import DumpMode, OutputsStructure
from evalscope.utils.logger import get_logger

logger = get_logger()

TEST_LEVEL_LIST = [0, 1]

# Example: export TEST_LEVEL_LIST=0,1
TEST_LEVEL_LIST_STR = 'TEST_LEVEL_LIST'


def test_level_list():
    global TEST_LEVEL_LIST
    if TEST_LEVEL_LIST_STR in os.environ:
        TEST_LEVEL_LIST = [int(x) for x in os.environ[TEST_LEVEL_LIST_STR].split(',')]

    return TEST_LEVEL_LIST


def jsonl_to_list(jsonl_file):
    """
    Read jsonl file to list.

    Args:
        jsonl_file: jsonl file path.

    Returns:
        list: list of lines. Each line is a dict.
    """
    res_list = []
    with jsonl.open(jsonl_file, mode='r') as reader:
        for line in reader.iter(type=dict, allow_none=True, skip_invalid=False):
            res_list.append(line)
    return res_list


def jsonl_to_reader(jsonl_file):
    """
    Read jsonl file to reader object.

    Args:
        jsonl_file: jsonl file path.

    Returns:
        reader: jsonl reader object.
    """
    with jsonl.open(jsonl_file, mode='r') as reader:
        return reader


def jsonl_to_csv():
    pass


def dump_jsonl_data(data_list, jsonl_file, dump_mode=DumpMode.OVERWRITE):
    """
    Dump data to jsonl file.

    Args:
        data_list: data list to be dumped.  [{'a': 'aaa'}, ...]
        jsonl_file: jsonl file path.
        dump_mode: dump mode. It can be 'overwrite' or 'append'.
    """
    if not jsonl_file:
        raise ValueError('output file must be provided.')

    jsonl_file = os.path.expanduser(jsonl_file)

    if dump_mode == DumpMode.OVERWRITE:
        dump_mode = 'w'
    elif dump_mode == DumpMode.APPEND:
        dump_mode = 'a'
    with jsonl.open(jsonl_file, mode=dump_mode) as writer:
        writer.write_all(data_list)
    logger.info(f'Dump data to {jsonl_file} successfully.')


def yaml_to_dict(yaml_file) -> dict:
    """
    Read yaml file to dict.
    """
    with open(yaml_file, 'r') as f:
        try:
            stream = yaml.safe_load(f)
        except yaml.YAMLError as e:
            logger.error(f'{e}')
            raise e

    return stream


def dict_to_yaml(d: dict, yaml_file: str):
    """
    Dump dict to yaml file.
    """
    with open(yaml_file, 'w') as f:
        yaml.dump(d, f, default_flow_style=False)
    logger.info(f'Dump data to {yaml_file} successfully.')


def json_to_dict(json_file) -> dict:
    """
    Read json file to dict.
    """
    with open(json_file, 'r') as f:
        try:
            stream = json.load(f)
        except json.JSONDecodeError as e:
            logger.error(f'{e}')
            raise e

    return stream


def get_obj_from_cfg(eval_class_ref: Any, *args, **kwargs) -> Any:
    module_name, spliter, cls_name = eval_class_ref.partition(':')

    try:
        obj_cls = importlib.import_module(module_name)
    except ImportError as e:
        logger.error(f'{e}')
        raise e

    if spliter:
        for attr in cls_name.split('.'):
            obj_cls = getattr(obj_cls, attr)

    return functools.partial(obj_cls, *args, **kwargs)


def markdown_table(header_l, data_l):
    md_str = f'| {" | ".join(header_l)} |'
    md_str += f'\n| {" | ".join(["---"] * len(header_l))} |'
    for data in data_l:
        if isinstance(data, str):
            data = [data]
        assert len(data) <= len(header_l)
        tmp = data + [''] * (len(header_l) - len(data))
        md_str += f'\n| {" | ".join(tmp)} |'
    return md_str


def random_seeded_choice(seed: Union[int, str, float], choices, **kwargs):
    """Random choice with a (potentially string) seed."""
    return random.Random(seed).choices(choices, k=1, **kwargs)[0]


def gen_hash(name: str):
    return hashlib.md5(name.encode(encoding='UTF-8')).hexdigest()


def dict_torch_dtype_to_str(d: Dict[str, Any]) -> dict:
    """
        Checks whether the passed dictionary and its nested dicts have a *torch_dtype* key and if it's not None,
        converts torch.dtype to a string of just the type. For example, `torch.float32` get converted into *"float32"*
        string, which can then be stored in the json format.

        Refer to: https://github.com/huggingface/transformers/pull/16065/files for details.
        """
    if d.get('torch_dtype', None) is not None and not isinstance(d['torch_dtype'], str):
        d['torch_dtype'] = str(d['torch_dtype']).split('.')[1]

    for value in d.values():
        if isinstance(value, dict):
            dict_torch_dtype_to_str(value)

    return d


class ResponseParser:

    @staticmethod
    def parse_first_capital(text: str) -> str:
        for t in text:
            if t.isupper():
                return t
        return ''

    @staticmethod
    def parse_last_capital(text: str) -> str:
        for t in text[::-1]:
            if t.isupper():
                return t
        return ''

    @staticmethod
    def parse_first_option_with_choices(text: str, options: list) -> str:
        """
        Find first valid option for text.

        Args:
            text: The text to parse.
            options: The options to find. e.g. ['A', 'B', 'C', 'D']
        """
        options_concat = '|'.join([str(i) for i in options])

        patterns = [
            f'答案是?\s?([{options_concat}])',
            f'答案是?\s?：([{options_concat}])',
            f'答案是?\s?:([{options_concat}])',
            f'答案应该?是\s?([{options_concat}])',
            f'答案应该?选\s?([{options_concat}])',
            f'答案为\s?([{options_concat}])',
            f'答案选\s?([{options_concat}])',
            f'选择?\s?([{options_concat}])',
            f'故选?\s?([{options_concat}])'
            f'只有选?项?\s?([{options_concat}])\s?是?对',
            f'只有选?项?\s?([{options_concat}])\s?是?错',
            f'只有选?项?\s?([{options_concat}])\s?不?正确',
            f'只有选?项?\s?([{options_concat}])\s?错误',
            f'说法不?对选?项?的?是\s?([{options_concat}])',
            f'说法不?正确选?项?的?是\s?([{options_concat}])',
            f'说法错误选?项?的?是\s?([{options_concat}])',
            f'([{options_concat}])\s?是正确的',
            f'([{options_concat}])\s?是正确答案',
            f'选项\s?([{options_concat}])\s?正确',
            f'所以答\s?([{options_concat}])',
            f'1.\s?([{options_concat}])[.。$]?$',
            f'所以\s?([{options_concat}][.。$]?$)',
            f'所有\s?([{options_concat}][.。$]?$)',
            f'[\s，：:,]([{options_concat}])[。，,\.]?$',
            f'[\s，,：:][故即]([{options_concat}])[。\.]?$',
            f'[\s，,：:]因此([{options_concat}])[。\.]?$',
            f'[是为。]\s?([{options_concat}])[。\.]?$',
            f'因此\s?([{options_concat}])[。\.]?$',
            f'显然\s?([{options_concat}])[。\.]?$',
            f'答案是\s?(\S+)(?:。|$)',
            f'答案应该是\s?(\S+)(?:。|$)',
            f'答案为\s?(\S+)(?:。|$)',
            f'答案是(.*?)[{options_concat}]',
            f'答案为(.*?)[{options_concat}]',
            f'固选(.*?)[{options_concat}]',
            f'答案应该是(.*?)[{options_concat}]',
            f'[Tt]he answer is [{options_concat}]',
            f'[Tt]he correct answer is [{options_concat}]',
            f'[Tt]he correct answer is:\n[{options_concat}]',
            f'(\s|^)[{options_concat}][\s。，,\.$]',  # noqa
            f'[{options_concat}]',
            f'^选项\s?([{options_concat}])',
            f'^([{options_concat}])\s?选?项',
            f'(\s|^)[{options_concat}][\s。，,：:\.$]',
            f'(\s|^)[{options_concat}](\s|$)',
            f'1.\s?(.*?)$',
        ]

        regexes = [re.compile(pattern) for pattern in patterns]
        for regex in regexes:
            match = regex.search(text)
            if match:
                outputs = match.group(0)
                for i in options:
                    if i in outputs:
                        return i
        return ''

    @staticmethod
    def parse_first_option(text: str) -> str:
        """
        Find first valid option for text.

        Args:
            text: The text to parse.
        """
        patterns = [
            r'[Aa]nswer:\s*(\w+)',
            r'[Tt]he correct answer is:\s*(\w+)',
            r'[Tt]he correct answer is:\n\s*(\w+)',
            r'[Tt]he correct answer is:\n\n-\s*(\w+)',
            r'[Tt]he answer might be:\n\n-\s*(\w+)',
            r'[Tt]he answer is \s*(\w+)',
        ]

        regexes = [re.compile(pattern) for pattern in patterns]
        for regex in regexes:
            match = regex.search(text)
            if match:
                return match.group(1)
        return ''

    @staticmethod
    def parse_first_capital_multi(text: str) -> str:
        match = re.search(r'([A-D]+)', text)
        if match:
            return match.group(1)
        return ''

    @staticmethod
    def parse_last_option(text: str, options: str) -> str:
        match = re.findall(rf'([{options}])', text)
        if match:
            return match[-1]
        return ''


def make_outputs_dir(root_dir: str, datasets: list, model_id: str, model_revision: str):
    # model_revision = model_revision if model_revision is not None else 'none'
    # now = datetime.datetime.now()
    # format_time = now.strftime('%Y%m%d_%H%M%S')
    # outputs_name = format_time + '_' + 'default' + '_' + model_id.replace('/', '_') + '_' + model_revision
    # outputs_dir = os.path.join(work_dir, outputs_name)
    # dataset_name = dataset_id.replace('/', '_')
    # outputs_dir = os.path.join(work_dir, dataset_name)

    if not model_id:
        model_id = 'default'
    model_id = model_id.replace('/', '_')

    if not model_revision:
        model_revision = 'default'

    outputs_dir = os.path.join(root_dir, f"eval_{'-'.join(datasets)}_{model_id}_{model_revision}")

    return outputs_dir


def process_outputs_structure(outputs_dir: str, is_make: bool = True) -> dict:
    logs_dir = os.path.join(outputs_dir, 'logs')
    predictions_dir = os.path.join(outputs_dir, 'predictions')
    reviews_dir = os.path.join(outputs_dir, 'reviews')
    reports_dir = os.path.join(outputs_dir, 'reports')
    configs_dir = os.path.join(outputs_dir, 'configs')

    if is_make:
        os.makedirs(outputs_dir, exist_ok=True)
        os.makedirs(logs_dir, exist_ok=True)
        os.makedirs(predictions_dir, exist_ok=True)
        os.makedirs(reviews_dir, exist_ok=True)
        os.makedirs(reports_dir, exist_ok=True)
        os.makedirs(configs_dir, exist_ok=True)

    outputs_structure = {
        OutputsStructure.LOGS_DIR: logs_dir,
        OutputsStructure.PREDICTIONS_DIR: predictions_dir,
        OutputsStructure.REVIEWS_DIR: reviews_dir,
        OutputsStructure.REPORTS_DIR: reports_dir,
        OutputsStructure.CONFIGS_DIR: configs_dir,
    }

    return outputs_structure


def import_module_util(import_path_prefix: str, module_name: str, members_to_import: list) -> dict:
    """
    Import module utility function.

    Args:
        import_path_prefix: e.g. 'evalscope.benchmarks.'
        module_name: The module name to import. e.g. 'mmlu'
        members_to_import: The members to import.
            e.g. ['DATASET_ID', 'SUBJECT_MAPPING', 'SUBSET_LIST', 'DataAdapterClass']

    Returns:
        dict: imported modules map. e.g. {'DATASET_ID': 'mmlu', 'SUBJECT_MAPPING': {...}, ...}
    """
    imported_modules = {}
    module = importlib.import_module(import_path_prefix + module_name)
    for member_name in members_to_import:
        imported_modules[member_name] = getattr(module, member_name)

    return imported_modules


def normalize_score(score: Union[float, dict], keep_num: int = 4) -> Union[float, dict]:
    """
    Normalize score.

    Args:
        score: input score, could be float or dict. e.g. 0.12345678 or {'acc': 0.12345678, 'f1': 0.12345678}
        keep_num: number of digits to keep.

    Returns:
        Union[float, dict]: normalized score. e.g. 0.1234 or {'acc': 0.1234, 'f1': 0.1234}
    """
    if isinstance(score, float):
        score = round(score, keep_num)
    elif isinstance(score, dict):
        score = {k: round(v, keep_num) for k, v in score.items()}
    else:
        logger.warning(f'Unknown score type: {type(score)}')

    return score


def split_str_parts_by(text: str, delimiters: List[str]):
    """Split the text field into parts.
    Args:
        text: A text to be split.
        delimiters: The delimiters.
    Returns:
        The split text in list of dicts.
    """
    all_start_chars = [d[0] for d in delimiters]
    all_length = [len(d) for d in delimiters]

    text_list = []
    last_words = ''

    while len(text) > 0:
        for char_idx, char in enumerate(text):
            match_index = [idx for idx, start_char in enumerate(all_start_chars) if start_char == char]
            is_delimiter = False
            for index in match_index:
                if text[char_idx:char_idx + all_length[index]] == delimiters[index]:
                    if last_words:
                        if text_list:
                            text_list[-1]['content'] = last_words
                        else:
                            text_list.append({'key': '', 'content': last_words})
                    last_words = ''
                    text_list.append({'key': delimiters[index]})
                    text = text[char_idx + all_length[index]:]
                    is_delimiter = True
                    break
            if not is_delimiter:
                last_words += char
            else:
                break
        if last_words == text:
            text = ''

    text_list[-1]['content'] = last_words
    return text_list


def calculate_loss_scale(response: str, use_loss_scale=False) -> Tuple[List[str], List[float]]:
    """Calculate the loss scale by splitting the agent response.
    This algorithm comes from paper: https://arxiv.org/pdf/2309.00986.pdf
    Agent response format:
    ```text
        Thought: you should always think about what to do
        Action: the action to take, should be one of the above tools[fire_recognition,
            fire_alert, call_police, call_fireman]
        Action Input: the input to the action
        Observation: the result of the action
        ... (this Thought/Action/Action Input/Observation can be repeated zero or more times)
        Thought: I now know the final answer
        Final Answer: the final answer to the original input question
    ```
    Args:
        response: The response text
        use_loss_scale: Use weighted loss. With this, some part of the loss will be enhanced to improve performance.
    Returns:
        A tuple of agent response parts and their weights.
    """
    if 'Action:' in response and 'Observation:' in response and use_loss_scale:
        agent_keyword = ['Action:', 'Action Input:', 'Thought:', 'Final Answer:', 'Observation:']
        agent_parts = split_str_parts_by(response, agent_keyword)
        weights = []
        agent_content = []
        for c in agent_parts:
            if c['key'] in ('Action:', 'Action Input:'):
                weights += [2.0]
                weights += [2.0]
            elif c['key'] in ('Thought:', 'Final Answer:', ''):
                weights += [1.0]
                weights += [1.0]
            elif c['key'] in ('Observation:', ):
                weights += [2.0]
                weights += [0.0]
            agent_content.append(c['key'])
            agent_content.append(c['content'])
        return agent_content, weights
    else:
        return [response], [1.0]


def get_bucket_sizes(max_length: int) -> List[int]:
    return [max_length // 4 * (i + 1) for i in range(4)]


def _get_closet_bucket(bucket_sizes, data_length):
    """Select the one from bucket_sizes that is closest in distance to
    data_length. This is required for TorchAcc.
    """
    cloest_length = sys.maxsize
    for b in bucket_sizes:
        if b == data_length or ((b < cloest_length) and (b > data_length)):
            cloest_length = b

    if cloest_length == sys.maxsize:
        bucket_sizes.append(data_length)
        cloest_length = data_length

    return cloest_length


def pad_and_split_batch(padding_to, input_ids, attention_mask, labels, loss_scale, max_length, tokenizer, rank,
                        world_size):
    if padding_to is None:
        longest_len = input_ids.shape[-1]
        bucket_sizes = get_bucket_sizes(max_length)
        bucket_data_length = _get_closet_bucket(bucket_sizes, longest_len)
        padding_length = bucket_data_length - input_ids.shape[1]
        input_ids = F.pad(input_ids, (0, padding_length), 'constant', tokenizer.pad_token_id)
        attention_mask = F.pad(attention_mask, (0, padding_length), 'constant', 0)
        if loss_scale:
            loss_scale = F.pad(loss_scale, (0, padding_length), 'constant', 0.)
        labels = F.pad(labels, (0, padding_length), 'constant', -100)

    # manully split the batch to different DP rank.
    batch_size = input_ids.shape[0] // world_size
    if batch_size > 0:
        start = rank * batch_size
        end = (rank + 1) * batch_size
        input_ids = input_ids[start:end, :]
        attention_mask = attention_mask[start:end, :]
        labels = labels[start:end, :]
        if loss_scale:
            loss_scale = loss_scale[start:end, :]
    return input_ids, attention_mask, labels, loss_scale


def get_dist_setting() -> Tuple[int, int, int, int]:
    """return rank, local_rank, world_size, local_world_size"""
    rank = int(os.getenv('RANK', -1))
    local_rank = int(os.getenv('LOCAL_RANK', -1))
    world_size = int(os.getenv('WORLD_SIZE', 1))
    local_world_size = int(os.getenv('LOCAL_WORLD_SIZE', 1))
    return rank, local_rank, world_size, local_world_size


def use_torchacc() -> bool:
    return os.getenv('USE_TORCHACC', '0') == '1'


def is_module_installed(module_name):
    try:
        importlib.import_module(module_name)
        return True
    except ImportError:
        return False


def get_module_path(module_name):
    spec = importlib.util.find_spec(module_name)
    if spec and spec.origin:
        return os.path.abspath(spec.origin)
    else:
        raise ValueError(f'Cannot find module: {module_name}')


def get_valid_list(input_list, candidate_list):
    """
    Get the valid and invalid list from input_list based on candidate_list.
    Args:
        input_list: The input list.
        candidate_list: The candidate list.

    Returns:
        valid_list: The valid list.
        invalid_list: The invalid list.
    """
    return [i for i in input_list if i in candidate_list], \
           [i for i in input_list if i not in candidate_list]


def get_latest_folder_path(work_dir):
    from datetime import datetime
    # Get all subdirectories in the work_dir
    folders = [f for f in os.listdir(work_dir) if os.path.isdir(os.path.join(work_dir, f))]

    # Get the timestamp（YYYYMMDD_HHMMSS）
    timestamp_pattern = re.compile(r'^\d{8}_\d{6}$')

    # Filter out the folders
    timestamped_folders = [f for f in folders if timestamp_pattern.match(f)]

    if not timestamped_folders:
        print(f'>> No timestamped folders found in {work_dir}!')
        return None

    # timestamp parser
    def parse_timestamp(folder_name):
        return datetime.strptime(folder_name, '%Y%m%d_%H%M%S')

    # Find the latest folder
    latest_folder = max(timestamped_folders, key=parse_timestamp)

    return os.path.join(work_dir, latest_folder)


def csv_to_list(file_path: str) -> List[dict]:
    import csv

    with open(file_path, mode='r', newline='', encoding='utf-8') as csv_file:
        csv_reader = csv.DictReader(csv_file)
        result = [row for row in csv_reader]

    return result


def seed_everything(seed: int):
    """Set all random seeds to a fixed value for reproducibility.

    Args:
        seed (int): The seed value.
    """
    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    if torch.cuda.is_available():
        torch.cuda.manual_seed_all(seed)
        torch.backends.cudnn.deterministic = True
        torch.backends.cudnn.benchmark = False



import os
from typing import Any, Dict, Iterator, List

import json

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.api.base import ApiPluginBase
from evalscope.perf.plugin.registry import register_api
from evalscope.utils.logger import get_logger

logger = get_logger()


@register_api('dashscope')
class DashScopeApiPlugin(ApiPluginBase):

    def __init__(self, mode_path: str):
        """Init the plugin

        Args:
            mode_path (str): The model path, we use the tokenizer
                weight in the model to calculate the number of the
                input and output tokens.
        """
        super().__init__(model_path=mode_path)

    def build_request(self, messages: List[Dict], param: Arguments) -> Dict:
        """Build the openai format request based on prompt, dataset

        Args:
            messages (List[Dict]): The basic message to generator query.
            param (QueryParameters): The query parameters.

        Raises:
            Exception: NotImplemented

        Returns:
            Dict: The request body. None if prompt format is error.
        """
        try:
            if param.query_template is not None:
                if param.query_template.startswith('@'):
                    file_path = param.query_template[1:]
                    if os.path.exists(file_path):
                        with open(file_path, 'r') as file:
                            query = json.load(file)
                    else:
                        raise FileNotFoundError(f'{file_path}')
                else:
                    query = json.loads(param.query_template)

                query['input']['messages'] = messages  # replace template content with message.
                return self.__compose_query_from_parameter(query, param)
            else:
                query = {'messages': messages}
                return self.__compose_query_from_parameter(query, param)
        except Exception as e:
            logger.exception(e)
            return None

    def __compose_query_from_parameter(self, payload: Dict, param: Arguments):
        payload['model'] = param.model
        if 'parameters' not in payload:
            payload['parameters'] = {}
        if param.max_tokens is not None:
            payload['parameters']['max_tokens'] = param.max_tokens
        if param.frequency_penalty is not None:
            payload['parameters']['frequency_penalty'] = param.frequency_penalty
        if param.logprobs is not None:
            payload['parameters']['logprobs'] = param.logprobs
        if param.n_choices is not None:
            payload['parameters']['n'] = param.n_choices
        if param.seed is not None:
            payload['parameters']['seed'] = param.seed
        if param.stop is not None:
            payload['parameters']['stop'] = param.stop
        if param.stream is not None and not param.stream:
            payload['parameters']['stream'] = param.stream
        if param.temperature is not None:
            payload['parameters']['temperature'] = param.temperature
        if param.top_p is not None:
            payload['parameters']['top_p'] = param.top_p
        return payload

    def parse_responses(self, responses, **kwargs) -> Dict:
        """Parser responses and return number of request and response tokens.

        Args:
            responses (List[bytes]): List of http response body, for stream output,
                there are multiple responses, for general only one.
            kwargs: (Any): The command line --parameter content.

        Returns:
            Tuple: Return number of prompt token and number of completion tokens.
        """
        last_response = responses[-1]
        js = json.loads(last_response)
        return js['usage']['input_tokens'], js['usage']['output_tokens']



# Copyright (c) Alibaba, Inc. and its affiliates.

import subprocess
import unittest
from evalscope.utils import test_level_list, is_module_installed
from evalscope.utils.logger import get_logger

logger = get_logger()


class TestRun(unittest.TestCase):

    def setUp(self) -> None:
        logger.info(f'Init env for evalscope native run UTs ...\n')
        self._check_env('evalscope')

    def tearDown(self) -> None:
        pass

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f'{module_name} is installed.')
        else:
            raise ModuleNotFoundError(f'run: pip install {module_name}')

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_simple_eval(self):
        model = 'ZhipuAI/chatglm3-6b'
        template_type = 'chatglm3'
        datasets = 'arc'  # arc ceval
        limit = 100

        cmd_simple = f'python3 -m evalscope.run ' \
                     f'--model {model} ' \
                     f'--template-type {template_type} ' \
                     f'--datasets {datasets} ' \
                     f'--limit {limit}'

        logger.info(f'Start to run command: {cmd_simple}')
        run_res = subprocess.run(cmd_simple, text=True, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        assert run_res.returncode == 0, f'Failed to run command: {cmd_simple}'
        logger.info(f'>>test_run_simple_eval stdout: {run_res.stdout}')
        logger.error(f'>>test_run_simple_eval stderr: {run_res.stderr}')

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_eval_with_args(self):
        model = 'ZhipuAI/chatglm3-6b'
        template_type = 'chatglm3'
        datasets = 'arc ceval'  # arc ceval
        limit = 5
        dataset_args = '{"ceval": {"few_shot_num": 0, "few_shot_random": false}}'

        cmd_with_args = f'python3 -m evalscope.run ' \
                        f'--model {model} ' \
                        f'--template-type {template_type} ' \
                        f'--datasets {datasets} ' \
                        f'--limit {limit} ' \
                        f'--generation-config do_sample=false,temperature=0.0 ' \
                        f"""--dataset-args \'{dataset_args}\' """

        logger.info(f'Start to run command: {cmd_with_args}')
        run_res = subprocess.run(cmd_with_args, text=True, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        assert run_res.returncode == 0, f'Failed to run command: {cmd_with_args}'
        logger.info(f'>>test_run_eval_with_args stdout: {run_res.stdout}')
        logger.error(f'>>test_run_eval_with_args stderr: {run_res.stderr}')

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_eval_local(self):
        ...


if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass, field
from typing import List, Dict


@dataclass
class Arguments:
    # fmt: off
    """
    A dataclass to store and manage the arguments for the model configuration and data processing.
    """
    """
    For CLIP model support, you can use the following fields:
        model_name: str
        revision: str = "master"
        hub: str = "modelscope"
        
    For API VLM model support, you can use the following fields, (image caption only):
        model_name="gpt-4o-mini"
        api_base: str = "",
        api_key: Optional[str] = None
        prompt: str = None
    """
    models: List[Dict] = field(default_factory=dict)  # List of paths to the pre-trained models or model identifiers
    dataset_name: List[str] = field(default_factory=list)  # List of dataset names to be used
    data_dir: str = None  # Root directory where the datasets are stored
    split: str = "test"  # Split of the dataset to be used (e.g., 'train', 'validation', 'test')
    task: str = None
    batch_size: int = 128  # Batch size for data loading
    num_workers: int = 1  # Number of workers for data loading
    verbose: bool = True  # Flag to enable verbose logging
    output_dir: str = "outputs"  # Directory where the outputs (e.g., predictions, logs) will be saved
    cache_dir: str = "cache"  # Directory where the dataset cache will be stored
    skip_existing: bool = False  # Flag to skip processing if outputs already exist
    limit: int = None # Limit the number of samples to be processed



# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.truthful_qa.truthful_qa_adapter import TruthfulQaAdapter, DATASET_ID, SUBSET_LIST
from evalscope.benchmarks.truthful_qa.truthful_qa_adapter import TruthfulQaAdapter as DataAdapterClass
from evalscope.models.model_adapter import ContinuationLogitsModelAdapter as ModelAdapterClass    # noqa



# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.arc.arc_adapter import ARCAdapter, DATASET_ID, SUBSET_LIST
from evalscope.benchmarks.arc.arc_adapter import ARCAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa



# Copyright (c) Alibaba, Inc. and its affiliates.
from typing import Dict, Any, List

"""
The API meta template for OpenCompass.

See more details in the OpenCompass documentation: https://opencompass.org.cn/doc
Search for `meta template` in the documentation.
"""


class MetaTemplateType:

    default_api_meta_template_oc = 'default-api-meta-template-oc'

    @classmethod
    def get_template_name_list(cls) -> List[str]:
        res = []
        for k in cls.__dict__.keys():
            if k.startswith('__') or k == 'get_template_name_list':
                continue
            res.append(cls.__dict__[k])
        return res


TEMPLATE_MAPPING: Dict[str, Dict[str, Any]] = {}


def register_template(name: str,
                      template: Dict[str, Any],
                      exists_ok: bool = False):
    if not exists_ok and name in TEMPLATE_MAPPING:
        raise ValueError(f"The `{name}` has already been registered in the TEMPLATE_MAPPING.")

    TEMPLATE_MAPPING[name] = template


def get_template(name: str) -> Dict[str, Any]:
    if name not in TEMPLATE_MAPPING:
        raise ValueError(f"The `{name}` has not been registered in the TEMPLATE_MAPPING.")

    return TEMPLATE_MAPPING[name]


# Default API meta template for OpenCompass
register_template(
    name=MetaTemplateType.default_api_meta_template_oc,
    template=dict(
        round=[
            dict(role='HUMAN', api_role='HUMAN'),
            dict(role='BOT', api_role='BOT', generate=True)
        ],
        reserved_roles=[
            dict(role='SYSTEM', api_role='SYSTEM'),
        ],
    )
)


if __name__ == '__main__':
    res = MetaTemplateType.get_template_name_list()
    print(res)

    print(get_template(MetaTemplateType.default_api_meta_template_oc))



# Copyright (c) Alibaba, Inc. and its affiliates.


from evalscope.benchmarks.bbh.bbh_adapter import DATASET_ID, SUBSET_LIST
from evalscope.benchmarks.bbh.bbh_adapter import BBHAdapter as DataAdapterClass
from evalscope.models.model_adapter import ChatGenerationModelAdapter as ModelAdapterClass    # noqa



import typing as t

from pydantic import BaseModel
from ragas.prompt import PydanticPrompt, StringIO
from ragas.testset.persona import Persona


class PersonaGenerationPromptZH(PydanticPrompt[StringIO, Persona]):
    instruction: str = ('使用提供的摘要，生成一个可能会与内容互动或从中受益的角色。包括一个独特的名字和一个简洁的角色描述。')
    input_model: t.Type[StringIO] = StringIO
    output_model: t.Type[Persona] = Persona
    examples: t.List[t.Tuple[StringIO, Persona]] = [(
        StringIO(text='《数字营销指南》解释了在各种在线平台上吸引受众的策略。'),
        Persona(
            name='数字营销专家',
            role_description='专注于吸引受众并在线上提升品牌。',
        ),
    )]



import sys
from typing import Dict, Iterator, List

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.datasets.base import DatasetPluginBase
from evalscope.perf.plugin.registry import register_dataset


@register_dataset('line_by_line')
class LineByLineDatasetPlugin(DatasetPluginBase):
    """Read dataset and return prompt.
    """

    def __init__(self, query_parameters: Arguments):
        super().__init__(query_parameters)

    def build_messages(self) -> Iterator[List[Dict]]:
        for item in self.dataset_line_by_line(self.query_parameters.dataset_path):
            prompt = item.strip()
            if len(prompt) > self.query_parameters.min_prompt_length and len(
                    prompt) < self.query_parameters.max_prompt_length:
                yield [{'role': 'user', 'content': prompt}]



import base64
import os
import pickle
import sqlite3
import sys
from datetime import datetime

import json
from tabulate import tabulate

from evalscope.perf.arguments import Arguments
from evalscope.perf.utils.benchmark_util import BenchmarkData, BenchmarkMetrics
from evalscope.utils.logger import get_logger

logger = get_logger()


def encode_data(data) -> str:
    """Encodes data using base64 and pickle."""
    return base64.b64encode(pickle.dumps(data)).decode('utf-8')


def write_json_file(data, output_path):
    with open(output_path, 'w') as f:
        json.dump(data, f, indent=4, ensure_ascii=False)


def transpose_results(data):
    headers = data.keys()
    rows = zip(*data.values())

    return [dict(zip(headers, row)) for row in rows]


def create_result_table(cursor):
    cursor.execute('''CREATE TABLE IF NOT EXISTS result(
                      request TEXT,
                      start_time REAL,
                      chunk_times TEXT,
                      success INTEGER,
                      response_messages TEXT,
                      completed_time REAL,
                      latency REAL,
                      first_chunk_latency REAL,
                      n_chunks INTEGER,
                      chunk_time REAL,
                      prompt_tokens INTEGER,
                      completion_tokens INTEGER,
                      max_gpu_memory_cost REAL)''')


def insert_benchmark_data(cursor: sqlite3.Cursor, benchmark_data: BenchmarkData):
    request = encode_data(benchmark_data.request)
    chunk_times = json.dumps(benchmark_data.chunk_times)
    response_messages = encode_data(benchmark_data.response_messages)

    # Columns common to both success and failure cases
    common_columns = (
        request,
        benchmark_data.start_time,
        chunk_times,
        benchmark_data.success,
        response_messages,
        benchmark_data.completed_time,
    )

    if benchmark_data.success:
        # Add additional columns for success case
        additional_columns = (
            benchmark_data.query_latency,
            benchmark_data.first_chunk_latency,
            benchmark_data.n_chunks,
            benchmark_data.n_chunks_time,
            benchmark_data.prompt_tokens,
            benchmark_data.completion_tokens,
            benchmark_data.max_gpu_memory_cost,
        )
        query = """INSERT INTO result(
                      request, start_time, chunk_times, success, response_messages,
                      completed_time, latency, first_chunk_latency,
                      n_chunks, chunk_time, prompt_tokens, completion_tokens, max_gpu_memory_cost
                   ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"""
        cursor.execute(query, common_columns + additional_columns)
    else:
        query = """INSERT INTO result(
                      request, start_time, chunk_times, success, response_messages, completed_time
                   ) VALUES (?, ?, ?, ?, ?, ?)"""
        cursor.execute(query, common_columns)


def get_result_db_path(name, model):
    current_time = datetime.now().strftime('%Y%m%d_%H%M%S')
    output_dir = './outputs'
    result_db_path = os.path.join(output_dir, f'{name or model}_perf', current_time, 'benchmark_data.db')

    if not os.path.exists(os.path.dirname(result_db_path)):
        os.makedirs(os.path.dirname(result_db_path), exist_ok=True)

    logger.info(f'Save the result to: {result_db_path}')
    if os.path.exists(result_db_path):
        logger.warning('The db file exists, delete it and start again!.')
        sys.exit(1)

    return result_db_path


def get_percentile_results(result_db_path: str):

    def percentile_results(rows, index, percentiles):
        results = {}
        n_success_queries = len(rows)
        for percentile in percentiles:
            idx = int(n_success_queries * percentile / 100)
            row = rows[idx]
            value = row[index] if row[index] is not None else float('inf')
            results[percentile] = round(value, 4)
        return results

    query_sql = ('SELECT start_time, chunk_times, success, completed_time, latency, first_chunk_latency, '
                 'n_chunks, chunk_time, prompt_tokens, completion_tokens '
                 'FROM result WHERE success=1 ORDER BY first_chunk_latency ASC')
    percentiles = [10, 25, 50, 66, 75, 80, 90, 95, 98, 99]

    with sqlite3.connect(result_db_path) as con:
        rows = con.execute(query_sql).fetchall()

    if len(rows) <= len(percentiles):
        logger.info('Too little data to calculate quantiles!')
        return {}

    # Calculate percentiles for first chunk latency and latency
    first_chunk_latency_index = 5
    latency_index = 4

    first_chunk_latency_results = percentile_results(rows, first_chunk_latency_index, percentiles)
    rows.sort(key=lambda x: x[latency_index])
    latency_results = percentile_results(rows, latency_index, percentiles)

    # Prepare data for tabulation
    return {
        'Percentile': [f'{p}%' for p in percentiles],
        'First Chunk Latency (s)': [first_chunk_latency_results[p] for p in percentiles],
        'Latency (s)': [latency_results[p] for p in percentiles]
    }


def summary_result(args: Arguments, metrics: BenchmarkMetrics, expected_number_of_queries: int, result_db_path: str):
    result_path = os.path.dirname(result_db_path)
    write_json_file(args.to_dict(), os.path.join(result_path, 'benchmark_args.json'))

    data = metrics.create_message()
    data.update({'Expected number of requests': expected_number_of_queries, 'Result DB path': result_db_path})
    write_json_file(data, os.path.join(result_path, 'benchmark_summary.json'))

    # Print summary in a table
    table = tabulate(list(data.items()), headers=['Key', 'Value'], tablefmt='grid')
    logger.info('\nBenchmarking summary:\n' + table)

    # Get percentile results
    percentile_result = get_percentile_results(result_db_path)
    if percentile_result:
        write_json_file(transpose_results(percentile_result), os.path.join(result_path, 'benchmark_percentile.json'))
        # Print percentile results in a table
        table = tabulate(percentile_result, headers='keys', tablefmt='pretty')
        logger.info('\nPercentile results:\n' + table)

    if args.dataset.startswith('speed_benchmark'):
        speed_benchmark_result(result_db_path)


def speed_benchmark_result(result_db_path: str):
    query_sql = """
        SELECT
            prompt_tokens,
            ROUND(AVG(completion_tokens / latency), 2) AS avg_completion_token_per_second,
            ROUND(AVG(max_gpu_memory_cost), 2)
        FROM
            result
        WHERE
            success = 1 AND latency > 0
        GROUP BY
            prompt_tokens
    """

    with sqlite3.connect(result_db_path) as con:
        cursor = con.cursor()
        cursor.execute(query_sql)
        rows = cursor.fetchall()

    # Prepare data for tabulation
    headers = ['Prompt Tokens', 'Speed(tokens/s)', 'GPU Memory(GB)']
    data = [dict(zip(headers, row)) for row in rows]

    # Print results in a table
    table = tabulate(data, headers='keys', tablefmt='pretty')
    logger.info('\nSpeed Benchmark Results:\n' + table)

    # Write results to JSON file
    result_path = os.path.dirname(result_db_path)
    write_json_file(data, os.path.join(result_path, 'speed_benchmark.json'))



import argparse
import sys
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

import json


@dataclass
class Arguments:
    # Model and API
    model: str  # Model identifier
    attn_implementation: Optional[str] = None  # Attention implementaion, only for local inference
    api: str = 'openai'  # API to be used (default: 'openai')
    tokenizer_path: Optional[str] = None  # Path to the tokenizer

    # Connection settings
    url: str = 'http://127.0.0.1:8877/v1/chat/completions'  # URL for the API connection
    headers: Dict[str, Any] = field(default_factory=dict)  # Custom headers
    connect_timeout: int = 120  # Connection timeout in seconds
    read_timeout: int = 120  # Read timeout in seconds
    api_key: str = 'EMPTY'

    # Performance and parallelism
    number: Optional[int] = None  # Number of requests to be made
    parallel: int = 1  # Number of parallel requests
    rate: int = -1  # Rate limit for requests (default: -1, no limit)

    # Logging and debugging
    log_every_n_query: int = 10  # Log every N queries
    debug: bool = False  # Debug mode
    wandb_api_key: Optional[str] = None  # WandB API key for logging
    name: Optional[str] = None  # Name for the run

    # Prompt settings
    max_prompt_length: int = sys.maxsize  # Maximum length of the prompt
    min_prompt_length: int = 0  # Minimum length of the prompt
    prompt: Optional[str] = None  # The prompt text
    query_template: Optional[str] = None  # Template for the query

    # Dataset settings
    dataset: str = 'openqa'  # Dataset type (default: 'line_by_line')
    dataset_path: Optional[str] = None  # Path to the dataset

    # Response settings
    frequency_penalty: Optional[float] = None  # Frequency penalty for the response
    logprobs: Optional[bool] = None  # Whether to log probabilities
    max_tokens: Optional[int] = 2048  # Maximum number of tokens in the response
    min_tokens: Optional[int] = None  # Minimum number of tokens in the response
    n_choices: Optional[int] = None  # Number of response choices
    seed: Optional[int] = 42  # Random seed for reproducibility
    stop: Optional[List[str]] = field(default_factory=list)  # Stop sequences for the response
    stop_token_ids: Optional[List[str]] = field(default_factory=list)  # Stop token IDs for the response
    stream: Optional[bool] = None  # Whether to stream the response
    temperature: Optional[float] = None  # Temperature setting for the response
    top_p: Optional[float] = None  # Top-p (nucleus) sampling setting for the response

    @staticmethod
    def from_args(args):

        return Arguments(
            model=args.model,
            attn_implementation=args.attn_implementation,
            url=args.url,
            api_key=args.api_key,
            connect_timeout=args.connect_timeout,
            read_timeout=args.read_timeout,
            number=args.number,
            parallel=args.parallel,
            rate=args.rate,
            log_every_n_query=args.log_every_n_query,
            headers=args.headers,
            wandb_api_key=args.wandb_api_key,
            name=args.name,
            debug=args.debug,
            tokenizer_path=args.tokenizer_path,
            api=args.api,
            max_prompt_length=args.max_prompt_length,
            min_prompt_length=args.min_prompt_length,
            prompt=args.prompt,
            query_template=args.query_template,
            dataset=args.dataset,
            dataset_path=args.dataset_path,
            frequency_penalty=args.frequency_penalty,
            logprobs=args.logprobs,
            max_tokens=args.max_tokens,
            min_tokens=args.min_tokens,
            n_choices=args.n_choices,
            seed=args.seed,
            stop=args.stop,
            stop_token_ids=args.stop_token_ids,
            stream=args.stream,
            temperature=args.temperature,
            top_p=args.top_p)

    def __post_init__(self):
        self.headers = self.headers or {}  # Default to empty dictionary
        if self.api_key:
            # Assuming the API key is used as a Bearer token
            self.headers['Authorization'] = f'Bearer {self.api_key}'

    def __str__(self):
        return json.dumps(self.to_dict(), indent=4, default=str, ensure_ascii=False)

    def to_dict(self) -> Dict[str, Any]:
        return self.__dict__


class ParseKVAction(argparse.Action):

    def __call__(self, parser, namespace, values, option_string=None):
        if not values:
            setattr(namespace, self.dest, {})
        else:
            try:
                kv_dict = dict(kv.split('=') for kv in values)
                setattr(namespace, self.dest, kv_dict)
            except ValueError as e:
                parser.error(f'Error parsing key-value pairs: {e}')


def add_argument(parser: argparse.ArgumentParser):
    # yapf: disable
    # Model and API
    parser.add_argument('--model', type=str, required=True, help='The test model name.')
    parser.add_argument('--attn-implementation', required=False, default=None, help='Attention implementaion')
    parser.add_argument('--api', type=str, default='openai', help='Specify the service API')
    parser.add_argument(
        '--tokenizer-path', type=str, required=False, default=None, help='Specify the tokenizer weight path')

    # Connection settings
    parser.add_argument('--url', type=str, default='http://127.0.0.1:8877/v1/chat/completions')
    parser.add_argument('--headers', nargs='+', dest='headers', action=ParseKVAction, help='Extra HTTP headers')
    parser.add_argument('--api-key', type=str, required=False, default='EMPTY', help='The API key for authentication')
    parser.add_argument('--connect-timeout', type=int, default=120, help='The network connection timeout')
    parser.add_argument('--read-timeout', type=int, default=120, help='The network read timeout')

    # Performance and parallelism
    parser.add_argument('-n', '--number', type=int, default=None, help='How many requests to be made')
    parser.add_argument('--parallel', type=int, default=1, help='Set number of concurrency requests, default 1')
    parser.add_argument('--rate', type=int, default=-1, help='Number of requests per second. default None')

    # Logging and debugging
    parser.add_argument('--log-every-n-query', type=int, default=10, help='Logging every n query')
    parser.add_argument('--debug', action='store_true', default=False, help='Debug request send')
    parser.add_argument('--wandb-api-key', type=str, default=None, help='The wandb API key')
    parser.add_argument('--name', type=str, help='The wandb db result name and result db name')

    # Prompt settings
    parser.add_argument('--max-prompt-length', type=int, default=sys.maxsize, help='Maximum input prompt length')
    parser.add_argument('--min-prompt-length', type=int, default=0, help='Minimum input prompt length')
    parser.add_argument('--prompt', type=str, required=False, default=None, help='Specified the request prompt')
    parser.add_argument('--query-template', type=str, default=None, help='Specify the query template')

    # Dataset settings
    parser.add_argument('--dataset', type=str, default='openqa', help='Specify the dataset')
    parser.add_argument('--dataset-path', type=str, required=False, help='Path to the dataset file')

    # Response settings
    parser.add_argument('--frequency-penalty', type=float, help='The frequency_penalty value', default=None)
    parser.add_argument('--logprobs', action='store_true', help='The logprobs', default=None)
    parser.add_argument(
        '--max-tokens', type=int, help='The maximum number of tokens that can be generated', default=2048)
    parser.add_argument(
        '--min-tokens', type=int, help='The minimum number of tokens that can be generated', default=None)
    parser.add_argument('--n-choices', type=int, help='How many completion choices to generate', default=None)
    parser.add_argument('--seed', type=int, help='The random seed', default=42)
    parser.add_argument('--stop', nargs='*', help='The stop tokens', default=None)
    parser.add_argument('--stop-token-ids', nargs='*', help='Set the stop token IDs', default=None)
    parser.add_argument('--stream', action='store_true', help='Stream output with SSE', default=None)
    parser.add_argument('--temperature', type=float, help='The sample temperature', default=None)
    parser.add_argument('--top-p', type=float, help='Sampling top p', default=None)
    # yapf: enable


def parse_args():
    parser = argparse.ArgumentParser(description='Benchmark LLM service performance.')
    add_argument(parser)
    return parser.parse_args()



import os
from typing import Any, Dict, Iterator, List

import json
from transformers import AutoTokenizer

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.api.base import ApiPluginBase
from evalscope.perf.plugin.registry import register_api
from evalscope.utils.logger import get_logger

logger = get_logger()


@register_api(['openai', 'local_vllm', 'local'])
class OpenaiPlugin(ApiPluginBase):
    """Base of openai interface."""

    def __init__(self, mode_path: str):
        """Init the plugin

        Args:
            mode_path (str): The model path, we use the tokenizer
                weight in the model to calculate the number of the
                input and output tokens.
        """
        super().__init__(model_path=mode_path)
        if mode_path is not None:
            self.tokenizer = AutoTokenizer.from_pretrained(mode_path)
        else:
            self.tokenizer = None

    def build_request(self, messages: List[Dict] | str, param: Arguments) -> Dict:
        """Build the openai format request based on prompt, dataset

        Args:
            message (List[Dict] | str): The basic message to generator query.
            param (QueryParameters): The query parameters.

        Raises:
            Exception: NotImplemented

        Returns:
            Dict: The request body. None if prompt format is error.
        """
        try:
            if param.query_template is not None:
                if param.query_template.startswith('@'):
                    file_path = param.query_template[1:]
                    if os.path.exists(file_path):
                        with open(file_path, 'r') as file:
                            query = json.load(file)
                    else:
                        raise FileNotFoundError(f'{file_path}')
                else:
                    query = json.loads(param.query_template)

                if 'stream' in query.keys():
                    param.stream = query['stream']
                # replace template messages with input messages.
                query['messages'] = messages
            elif isinstance(messages, str):
                query = {'prompt': messages}
            else:
                query = {'messages': messages}
            return self.__compose_query_from_parameter(query, param)
        except Exception as e:
            logger.exception(e)
            return None

    def __compose_query_from_parameter(self, payload: Dict, param: Arguments):
        payload['model'] = param.model
        if param.max_tokens is not None:
            payload['max_tokens'] = param.max_tokens
        if param.min_tokens is not None:
            payload['min_tokens'] = param.min_tokens
        if param.frequency_penalty is not None:
            payload['frequency_penalty'] = param.frequency_penalty
        if param.logprobs is not None:
            payload['logprobs'] = param.logprobs
        if param.n_choices is not None:
            payload['n'] = param.n_choices
        if param.seed is not None:
            payload['seed'] = param.seed
        if param.stop is not None:
            payload['stop'] = param.stop
        if param.stream is not None and param.stream:
            payload['stream'] = param.stream
            payload['stream_options'] = {'include_usage': True}
        if param.stop_token_ids is not None:
            payload['stop_token_ids'] = param.stop_token_ids
        if param.temperature is not None:
            payload['temperature'] = param.temperature
        if param.top_p is not None:
            payload['top_p'] = param.top_p
        return payload

    def parse_responses(self, responses, request: Any = None, **kwargs) -> Dict:
        """Parser responses and return number of request and response tokens.
           sample of the output delta:
           {"id":"4","object":"chat.completion.chunk","created":1714030870,"model":"llama3","choices":[{"index":0,"delta":{"role":"assistant","content":""},"logprobs":null,"finish_reason":null}]}


        Args:
            responses (List[bytes]): List of http response body, for stream output,
                there are multiple responses, for general only one.
            kwargs: (Any): The command line --parameter content.
        Returns:
            Tuple: Return number of prompt token and number of completion tokens.
        """
        full_response_content = ''
        delta_contents = {}
        input_tokens = None
        output_tokens = None
        for response in responses:
            js = json.loads(response)
            if js['object'] == 'chat.completion':
                for choice in js['choices']:
                    delta_contents[choice['index']] = [choice['message']['content']]
                input_tokens = js['usage']['prompt_tokens']
                output_tokens = js['usage']['completion_tokens']
            elif js['object'] == 'text_completion':
                for choice in js['choices']:
                    delta_contents[choice['index']] = [choice['text']]
                input_tokens = js['usage']['prompt_tokens']
                output_tokens = js['usage']['completion_tokens']
            elif js['object'] == 'chat.completion.chunk':
                if 'choices' in js:
                    for choice in js['choices']:
                        if 'delta' in choice and 'index' in choice:
                            delta = choice['delta']
                            idx = choice['index']
                            if 'content' in delta:
                                delta_content = delta['content']
                                if idx in delta_contents:
                                    delta_contents[idx].append(delta_content)
                                else:
                                    delta_contents[idx] = [delta_content]
                # usage in chunk: {"id":"","object":"chat.completion.chunk","created":1718269986,"model":"llama3",
                # "choices":[],"usage":{"prompt_tokens":32,"total_tokens":384,"completion_tokens":352}}
                if 'usage' in js and js['usage']:
                    input_tokens = js['usage']['prompt_tokens']
                    output_tokens = js['usage']['completion_tokens']
        if (input_tokens is None and output_tokens is None and self.tokenizer is not None):
            input_tokens = 0
            output_tokens = 0
            for idx, choice_contents in delta_contents.items():
                full_response_content = ''.join([m for m in choice_contents])
                input_tokens += len(self.tokenizer.encode(request['messages'][0]['content']))
                output_tokens += len(self.tokenizer.encode(full_response_content))
        elif input_tokens is None and output_tokens is None:  # no usage info get.
            input_tokens = 0
            output_tokens = 0
            logger.warning('No usage information found. Please specify `--tokenizer-path` to generate usage details.')

        return input_tokens, output_tokens



# Copyright (c) Alibaba, Inc. and its affiliates.

import argparse
from evalscope.cli.start_perf import PerfBenchCMD


def run_cmd():
    parser = argparse.ArgumentParser(
        'EvalScope Command Line tool', usage='evalscope <command> [<args>]')
    subparsers = parser.add_subparsers(help='Performance benchmark command line.')
    
    PerfBenchCMD.define_args(subparsers)

    args = parser.parse_args()

    if not hasattr(args, 'func'):
        parser.print_help()
        exit(1)

    cmd = args.func(args)
    cmd.execute()
    # --url 'http://11.122.132.12:8000/v1/chat/completions' --parallel 1 --model 'qwen' --dataset 'datasets/LongAlpaca-12k.jsonl'  --log-every-n-query 1 --read-timeout=120  --parser 'openai.longalpaca_12k_qwen.py' -n 10 --max-prompt-length 128000 --tokenizer-path ''


if __name__ == '__main__':
    run_cmd()



import os
import subprocess
from contextlib import asynccontextmanager
from dataclasses import dataclass

import torch
import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from sse_starlette.sse import EventSourceResponse

from evalscope.perf.arguments import Arguments
from evalscope.perf.utils.chat_service import ChatCompletionRequest, ChatService, ModelList, TextCompletionRequest
from evalscope.utils.logger import get_logger

logger = get_logger()


@dataclass
class ServerSentEvent(object):

    def __init__(self, data='', event=None, id=None, retry=None):
        self.data = data
        self.event = event
        self.id = id
        self.retry = retry

    @classmethod
    def decode(cls, line):
        """Decode line to ServerSentEvent


        Args:
            line (str): The line.

        Return:
            ServerSentEvent (obj:`ServerSentEvent`): The ServerSentEvent object.

        """
        if not line:
            return None
        sse_msg = cls()
        # format data:xxx
        field_type, _, field_value = line.partition(':')
        if field_value.startswith(' '):  # compatible with openai api
            field_value = field_value[1:]
        if field_type == 'event':
            sse_msg.event = field_value
        elif field_type == 'data':
            field_value = field_value.rstrip()
            sse_msg.data = field_value
        elif field_type == 'id':
            sse_msg.id = field_value
        elif field_type == 'retry':
            sse_msg.retry = field_value
        else:
            pass

        return sse_msg


@asynccontextmanager
async def lifespan(app: FastAPI):
    yield
    if torch.cuda.is_available():
        torch.cuda.empty_cache()


def create_app(args) -> FastAPI:
    app = FastAPI(lifespan=lifespan)
    chat_service = ChatService(model_path=args.model, attn_implementation=args.attn_implementation)

    app.add_middleware(
        CORSMiddleware,
        allow_origins=['*'],
        allow_credentials=True,
        allow_methods=['*'],
        allow_headers=['*'],
    )

    @app.get('/v1/models', response_model=ModelList)
    async def list_models():
        return await chat_service.list_models()

    @app.post('/v1/completions')
    async def create_text_completion(request: TextCompletionRequest):
        return await chat_service._text_completion(request)

    @app.post('/v1/chat/completions')
    async def create_chat_completion(request: ChatCompletionRequest):
        if request.stream:
            return EventSourceResponse(chat_service._stream_chat(request))
        else:
            return await chat_service._chat(request)

    return app


def start_app(args: Arguments):
    if args.api == 'local':
        app = create_app(args)
        uvicorn.run(app, host='0.0.0.0', port=8877, workers=1)

    elif args.api == 'local_vllm':
        os.environ['VLLM_USE_MODELSCOPE'] = 'True'

        proc = subprocess.Popen([
            'python', '-m', 'vllm.entrypoints.openai.api_server', '--model', args.model, '--served-model-name',
            os.path.basename(args.model), '--tensor-parallel-size',
            str(torch.cuda.device_count()), '--max-model-len', '32768', '--gpu-memory-utilization', '0.9', '--host',
            '0.0.0.0', '--port', '8877', '--disable-log-requests', '--disable-log-stats'
        ])
        import atexit

        def on_exit():
            if proc.poll() is None:
                logger.info('Terminating the child process...')
                proc.terminate()
                try:
                    proc.wait(timeout=10)
                except subprocess.TimeoutExpired:
                    logger.warning('Child process did not terminate within the timeout, killing it forcefully...')
                    proc.kill()
                    proc.wait()
                logger.info('Child process terminated.')
            else:
                logger.info('Child process has already terminated.')

        atexit.register(on_exit)
    else:
        raise ValueError(f'Unknown API type: {args.api}')


if __name__ == '__main__':
    from collections import namedtuple

    args = namedtuple('Args', ['model', 'attn_implementation', 'api'])

    start_app(args(model='Qwen/Qwen2.5-0.5B-Instruct', attn_implementation=None, api='local_vllm'))



from abc import abstractmethod
from typing import Any, Dict, List, Tuple

from evalscope.perf.arguments import Arguments


class ApiPluginBase:

    def __init__(self, model_path: str) -> None:
        self.model_path = model_path

    @abstractmethod
    def build_request(self, messages: List[Dict], param: Arguments) -> Dict:
        """Build a api request body.

        Args:
            messages (List[Dict]): The messages generated by dataset.
            param (QueryParameters): The query parameters.

        Raises:
            NotImplementedError: Not implemented.

        Returns:
            Dict: The api request body.
        """
        raise NotImplementedError

    @abstractmethod
    def parse_responses(self, responses: List, request: Any = None, **kwargs: Any) -> Tuple[int, int]:
        """Parser responses and return number of request and response tokens.

        Args:
            responses (List[bytes]): List of http response body, for stream output,
                there are multiple responses, each is bytes, for general only one.
            request (Any): The request body.

        Returns:
            Tuple: (Number of prompt_tokens and number of completion_tokens).
        """
        raise NotImplementedError

    @staticmethod
    def replace_values(input_json: Any, model: str, prompt: str):
        if isinstance(input_json, dict):
            for key, value in input_json.items():
                if isinstance(value, str):
                    input_json[key] = value.replace('%m', model).replace('%p', prompt)
                else:
                    ApiPluginBase.replace_values(value, model, prompt)
        elif isinstance(input_json, list):
            for idx, item in enumerate(input_json):
                if isinstance(item, str):
                    input_json[idx] = item.replace('%m', model).replace('%p', prompt)
                else:
                    ApiPluginBase.replace_values(item, model, prompt)
        elif isinstance(input_json, str):
            input_json = input_json.replace('%m', model).replace('%p', prompt)
        else:
            pass



import importlib.util as iutil
import logging
from typing import Optional

init_loggers = {}

detailed_format = '%(asctime)s - %(name)s - %(filename)s - %(funcName)s - %(lineno)d - %(levelname)s - %(message)s'
simple_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

detailed_formatter = logging.Formatter(detailed_format)
simple_formatter = logging.Formatter(simple_format)

logging.basicConfig(format=simple_format, level=logging.INFO)


def get_logger(log_file: Optional[str] = None, log_level: int = logging.INFO, file_mode: str = 'w'):
    """Get logging logger

    Args:
        log_file: Log filename, if specified, file handler will be added to
            logger
        log_level: Logging level.
        file_mode: Specifies the mode to open the file, if filename is
            specified (if filemode is unspecified, it defaults to 'w').
    """

    logger_name = __name__.split('.')[0]
    logger = logging.getLogger(logger_name)
    logger.propagate = False

    if logger_name in init_loggers:
        if logger.level != log_level:
            logger.setLevel(log_level)
        add_file_handler_if_needed(logger, log_file, file_mode, log_level)
        for handler in logger.handlers:
            handler.setLevel(log_level)
            handler.setFormatter(detailed_formatter if log_level == logging.DEBUG else simple_formatter)
        return logger

    # handle duplicate logs to the console
    torch_dist = False
    is_worker0 = True
    if iutil.find_spec('torch') is not None:
        from modelscope.utils.torch_utils import is_dist, is_master

        torch_dist = is_dist()
        is_worker0 = is_master()

    if torch_dist:
        for handler in logger.root.handlers:
            if type(handler) is logging.StreamHandler:
                handler.setLevel(logging.ERROR)

    stream_handler = logging.StreamHandler()
    handlers = [stream_handler]

    if is_worker0 and log_file is not None:
        file_handler = logging.FileHandler(log_file, file_mode)
        handlers.append(file_handler)

    for handler in handlers:
        handler.setFormatter(detailed_formatter if log_level == logging.DEBUG else simple_formatter)
        handler.setLevel(log_level)
        logger.addHandler(handler)

    if is_worker0:
        logger.setLevel(log_level)
    else:
        logger.setLevel(logging.ERROR)

    init_loggers[logger_name] = True

    return logger


def add_file_handler_if_needed(logger, log_file, file_mode, log_level):
    for handler in logger.handlers:
        if isinstance(handler, logging.FileHandler):
            return

    if iutil.find_spec('torch') is not None:
        from modelscope.utils.torch_utils import is_master

        is_worker0 = is_master()
    else:
        is_worker0 = True

    if is_worker0 and log_file is not None:
        file_handler = logging.FileHandler(log_file, file_mode)
        file_handler.setFormatter(detailed_formatter if log_level == logging.DEBUG else simple_formatter)
        file_handler.setLevel(log_level)
        logger.addHandler(file_handler)



import asyncio
import os

import pandas as pd
from ragas.embeddings import LangchainEmbeddingsWrapper
from ragas.llms import LangchainLLMWrapper
from tqdm import tqdm

from evalscope.backend.rag_eval import LLM, ChatOpenAI, EmbeddingModel
from evalscope.backend.rag_eval.ragas.arguments import TestsetGenerationArguments
from evalscope.utils.logger import get_logger
from .translate_prompt import translate_prompts

logger = get_logger()


def get_transform(llm, embedding, language):
    """
    Creates and returns a default set of transforms for processing a knowledge graph.
    """
    from ragas.testset.transforms.engine import Parallel
    from ragas.testset.transforms.extractors import (
        EmbeddingExtractor,
        HeadlinesExtractor,
        SummaryExtractor,
    )
    from ragas.testset.transforms.extractors.llm_based import NERExtractor, ThemesExtractor
    from ragas.testset.transforms.relationship_builders import (
        CosineSimilarityBuilder,
        OverlapScoreBuilder,
    )
    from ragas.testset.transforms.splitters import HeadlineSplitter
    from ragas.testset.transforms.filters import CustomNodeFilter
    from ragas.testset.graph import NodeType
    from ragas.utils import num_tokens_from_string

    def summary_filter(node):
        return (node.type == NodeType.DOCUMENT and num_tokens_from_string(node.properties['page_content']) > 500)

    summary_extractor = SummaryExtractor(llm=llm, filter_nodes=lambda node: summary_filter(node))
    ner_extractor = NERExtractor(llm=llm, filter_nodes=lambda node: node.type == NodeType.CHUNK)
    theme_extractor = ThemesExtractor(llm=llm)
    headline_extractor = HeadlinesExtractor(llm=llm)

    asyncio.run(
        translate_prompts(
            prompts=[
                summary_extractor,
                theme_extractor,
                ner_extractor,
                headline_extractor,
            ],
            target_lang=language,
            llm=llm,
            adapt_instruction=True,
        ))

    splitter = HeadlineSplitter(min_tokens=500)

    summary_emb_extractor = EmbeddingExtractor(
        embedding_model=embedding,
        property_name='summary_embedding',
        embed_property_name='summary',
        filter_nodes=lambda node: summary_filter(node),
    )

    cosine_sim_builder = CosineSimilarityBuilder(
        property_name='summary_embedding',
        new_property_name='summary_similarity',
        threshold=0.7,
        filter_nodes=lambda node: summary_filter(node),
    )

    ner_overlap_sim = OverlapScoreBuilder(threshold=0.01, filter_nodes=lambda node: node.type == NodeType.CHUNK)

    node_filter = CustomNodeFilter(llm=llm, filter_nodes=lambda node: node.type == NodeType.CHUNK)

    transforms = [
        headline_extractor,
        splitter,
        summary_extractor,
        node_filter,
        Parallel(summary_emb_extractor, theme_extractor, ner_extractor),
        Parallel(cosine_sim_builder, ner_overlap_sim),
    ]

    return transforms


def get_distribution(llm, distribution, language):
    from ragas.testset.synthesizers.multi_hop import (
        MultiHopAbstractQuerySynthesizer,
        MultiHopSpecificQuerySynthesizer,
    )
    from ragas.testset.synthesizers.single_hop.specific import (
        SingleHopSpecificQuerySynthesizer, )

    single_hop = SingleHopSpecificQuerySynthesizer(llm=llm)
    multi_hop_abs = MultiHopAbstractQuerySynthesizer(llm=llm)
    multi_hop_spec = MultiHopSpecificQuerySynthesizer(llm=llm)

    asyncio.run(
        translate_prompts(
            prompts=[
                single_hop,
                multi_hop_abs,
                multi_hop_spec,
            ],
            target_lang=language,
            llm=llm,
            adapt_instruction=True,
        ))

    mapping = {
        'simple': single_hop,
        'multi_context': multi_hop_abs,
        'reasoning': multi_hop_spec,
    }

    return [(mapping[key], distribution[key]) for key in mapping if key in distribution]


def get_knowledge_graph(documents, transforms, local_file, run_config):
    from ragas.testset.graph import KnowledgeGraph, Node, NodeType
    from ragas.testset.transforms import apply_transforms

    if os.path.exists(local_file):
        logger.info(f'Loading knowledge graph from {local_file}')
        return KnowledgeGraph.load(local_file)
    # convert the documents to Ragas nodes
    nodes = []
    for doc in documents:
        node = Node(
            type=NodeType.DOCUMENT,
            properties={
                'page_content': doc.page_content,
                'document_metadata': doc.metadata,
            },
        )
        nodes.append(node)

    kg = KnowledgeGraph(nodes=nodes)

    # apply transforms and update the knowledge graph
    apply_transforms(kg, transforms, run_config=run_config)

    # save the knowledge graph
    output_path = os.path.dirname(local_file)
    os.makedirs(output_path, exist_ok=True)
    kg.save(local_file)
    logger.info(f'Knowledge graph saved to {local_file}')
    return kg


def get_persona(llm, kg, language):
    from evalscope.backend.rag_eval.ragas.prompts.persona_prompt import PersonaGenerationPromptZH
    from ragas.testset.persona import generate_personas_from_kg, PersonaGenerationPrompt
    from ragas.testset.graph import Node

    def filter(node: Node) -> bool:
        if (node.type.name == 'DOCUMENT' and node.properties.get('summary_embedding') is not None):
            return True
        else:
            return False

    if language == 'chinese':
        persona_prompt = PersonaGenerationPromptZH()
    else:
        persona_prompt = PersonaGenerationPrompt()
    # NOTE: can't translate this yet
    # asyncio.run(
    #     translate_prompts(
    #         prompts=[persona_prompt],
    #         target_lang=language,
    #         llm=llm,
    #         adapt_instruction=True,
    #     ))

    return generate_personas_from_kg(
        llm=llm,
        kg=kg,
        num_personas=3,
        persona_generation_prompt=persona_prompt,
        filter_fn=filter,
    )


def load_data(file_path):
    from langchain_community.document_loaders import UnstructuredFileLoader

    loader = UnstructuredFileLoader(file_path, mode='elements')
    data = loader.load()
    return data


def generate_testset(args: TestsetGenerationArguments) -> None:

    from ragas.testset import TestsetGenerator
    from ragas import RunConfig

    # load data
    documents = load_data(args.docs)

    # generator with models
    generator_llm = LLM.load(**args.generator_llm)
    embeddings = EmbeddingModel.load(**args.embeddings)

    wrapped_llm = LangchainLLMWrapper(generator_llm)
    wrapped_embeddings = LangchainEmbeddingsWrapper(embeddings)

    # Change resulting question type distribution
    distributions = get_distribution(wrapped_llm, args.distribution, args.language)

    run_config = RunConfig(timeout=600, max_retries=3, max_wait=120, max_workers=1, log_tenacity=True)
    # get transforms
    transforms = get_transform(
        wrapped_llm,
        wrapped_embeddings,
        args.language,
    )

    # get knowledge graph
    knowledge_graph = get_knowledge_graph(documents, transforms, args.knowledge_graph, run_config)

    persona_list = get_persona(llm=wrapped_llm, kg=knowledge_graph, language=args.language)

    generator = TestsetGenerator(llm=wrapped_llm, knowledge_graph=knowledge_graph, persona_list=persona_list)

    testset = generator.generate(
        testset_size=args.test_size,
        query_distribution=distributions,
        run_config=run_config,
        with_debugging_logs=True,
        raise_exceptions=True,
    )

    # save file
    testset_df = testset.to_pandas()
    output_path = os.path.dirname(args.output_file)
    os.makedirs(output_path, exist_ok=True)
    testset_df.to_json(args.output_file, indent=4, index=False, orient='records', force_ascii=False)

    # get answer
    testset_with_answer = get_answer(testset_df, generator_llm, args.language)
    testset_with_answer.to_json(
        args.output_file.replace('.json', '_with_answer.json'),
        indent=4,
        index=False,
        orient='records',
        force_ascii=False,
    )


def get_answer(testset_df, generator_llm, language: None):
    template = """You are an assistant for question-answering tasks.
Use the following pieces of retrieved context to answer the question.
If you don't know the answer, just say that you don't know. Answer in {language}.
Question: {question}
Context: {contexts}
Answer:
"""

    items = []
    for i in tqdm(range(len(testset_df)), desc='Generating Answers'):
        row = testset_df.iloc[i]
        question = row['user_input']
        contexts = '\n'.join(row['reference_contexts'])

        # Combine question and contexts as input for the LLM
        input_text = template.format(language=language, question=question, contexts=contexts)

        # Generate the answer using the generator LLM
        answer = generator_llm.invoke(input_text)
        if isinstance(generator_llm, ChatOpenAI):
            answer = answer.content
        items.append({
            'user_input': question,
            'retrieved_contexts': row['reference_contexts'],
            'response': answer,
            'reference': row['reference'],
        })

    return pd.DataFrame.from_dict(items)



from collections import defaultdict
from typing import List
from mteb import AbsTask
from datasets import DatasetDict
from modelscope import MsDataset
import datasets
from evalscope.backend.rag_eval.cmteb.tasks import CLS_DICT, CLS_RETRIEVAL, CLS_CUSTOM

__all__ = ["TaskBase"]


class TaskBase:

    @staticmethod
    def get_tasks(task_names, **kwargs) -> List[AbsTask]:

        return [TaskBase.get_task(task_name, **kwargs) for task_name in task_names]

    @staticmethod
    def get_task(task_name, **kwargs) -> AbsTask:

        if task_name in CLS_CUSTOM:
            task_cls = CLS_CUSTOM[task_name]
        elif task_name in CLS_DICT:
            task_cls = CLS_DICT[task_name]
            task_cls.load_data = load_data
        else:
            from mteb.overview import TASKS_REGISTRY

            task_cls = TASKS_REGISTRY[task_name]
            if task_cls.metadata.type != "Retrieval":
                task_cls.load_data = load_data

        # init task instance
        task_instance = task_cls(**kwargs)
        return task_instance


def load_data(self, **kwargs):
    """Load dataset from the hub, compatible with ModelScope and Hugging Face."""
    if self.data_loaded:
        return

    limits = kwargs.get("limits", None)
    hub = kwargs.get("hub", "modelscope")
    name = self.metadata_dict.get("name")
    path = self.metadata_dict["dataset"].get("path")

    assert path is not None, "Path must be specified in dataset"

    # Loading the dataset based on the source hub
    if hub == "modelscope":
        import re

        path = re.sub(r"^mteb/", "MTEB/", path)
        dataset = MsDataset.load(path)
    else:
        dataset = datasets.load_dataset(**self.metadata_dict["dataset"])  # type: ignore

    if limits is not None:
        dataset = {
            split: dataset[split].select(range(min(limits, len(dataset[split]))))
            for split in dataset.keys()
        }

    if name in CLS_RETRIEVAL:
        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            dataset,
            path,
            self.metadata_dict["eval_splits"],
        )

    self.dataset = dataset
    self.dataset_transform()
    self.data_loaded = True


def load_retrieval_data(dataset, dataset_name: str, eval_splits: list) -> tuple:
    eval_split = eval_splits[0]
    qrels = MsDataset.load(dataset_name + "-qrels")[eval_split]

    corpus = {e["id"]: {"text": e["text"]} for e in dataset["corpus"]}
    queries = {e["id"]: e["text"] for e in dataset["queries"]}
    relevant_docs = defaultdict(dict)
    for e in qrels:
        relevant_docs[e["qid"]][e["pid"]] = e["score"]

    corpus = DatasetDict({eval_split: corpus})
    queries = DatasetDict({eval_split: queries})
    relevant_docs = DatasetDict({eval_split: relevant_docs})
    return corpus, queries, relevant_docs



# Copyright (c) Alibaba, Inc. and its affiliates.
from mmengine.config import read_base

with read_base():
    from opencompass.configs.datasets.mmlu.mmlu_gen_4d595a import mmlu_datasets
    from opencompass.configs.datasets.ceval.ceval_gen_5f30c7 import ceval_datasets
    from opencompass.configs.datasets.agieval.agieval_gen_64afd3 import agieval_datasets
    from opencompass.configs.datasets.GaokaoBench.GaokaoBench_gen_5cfe9e import GaokaoBench_datasets
    from opencompass.configs.datasets.humaneval.humaneval_gen_8e312c import humaneval_datasets
    from opencompass.configs.datasets.mbpp.mbpp_gen_830460 import mbpp_datasets
    from opencompass.configs.datasets.CLUE_C3.CLUE_C3_gen_8c358f import C3_datasets
    from opencompass.configs.datasets.CLUE_CMRC.CLUE_CMRC_gen_1bd3c8 import CMRC_datasets
    from opencompass.configs.datasets.CLUE_DRCD.CLUE_DRCD_gen_1bd3c8 import DRCD_datasets
    from opencompass.configs.datasets.CLUE_afqmc.CLUE_afqmc_gen_901306 import afqmc_datasets
    from opencompass.configs.datasets.CLUE_cmnli.CLUE_cmnli_gen_1abf97 import cmnli_datasets
    from opencompass.configs.datasets.CLUE_ocnli.CLUE_ocnli_gen_c4cb6c import ocnli_datasets
    from opencompass.configs.datasets.FewCLUE_bustm.FewCLUE_bustm_gen_634f41 import bustm_datasets
    from opencompass.configs.datasets.FewCLUE_chid.FewCLUE_chid_gen_0a29a2 import chid_datasets
    from opencompass.configs.datasets.FewCLUE_cluewsc.FewCLUE_cluewsc_gen_c68933 import cluewsc_datasets
    from opencompass.configs.datasets.FewCLUE_csl.FewCLUE_csl_gen_28b223 import csl_datasets
    from opencompass.configs.datasets.FewCLUE_eprstmt.FewCLUE_eprstmt_gen_740ea0 import eprstmt_datasets
    from opencompass.configs.datasets.FewCLUE_ocnli_fc.FewCLUE_ocnli_fc_gen_f97a97 import ocnli_fc_datasets
    from opencompass.configs.datasets.FewCLUE_tnews.FewCLUE_tnews_gen_b90e4a import tnews_datasets
    from opencompass.configs.datasets.lcsts.lcsts_gen_8ee1fe import lcsts_datasets
    from opencompass.configs.datasets.lambada.lambada_gen_217e11 import lambada_datasets
    from opencompass.configs.datasets.storycloze.storycloze_gen_7f656a import storycloze_datasets
    from opencompass.configs.datasets.SuperGLUE_AX_b.SuperGLUE_AX_b_gen_4dfefa import AX_b_datasets
    from opencompass.configs.datasets.SuperGLUE_AX_g.SuperGLUE_AX_g_gen_68aac7 import AX_g_datasets
    from opencompass.configs.datasets.SuperGLUE_BoolQ.SuperGLUE_BoolQ_gen_883d50 import BoolQ_datasets
    from opencompass.configs.datasets.SuperGLUE_CB.SuperGLUE_CB_gen_854c6c import CB_datasets
    from opencompass.configs.datasets.SuperGLUE_COPA.SuperGLUE_COPA_gen_91ca53 import COPA_datasets
    from opencompass.configs.datasets.SuperGLUE_MultiRC.SuperGLUE_MultiRC_gen_27071f import MultiRC_datasets
    from opencompass.configs.datasets.SuperGLUE_RTE.SuperGLUE_RTE_gen_68aac7 import RTE_datasets
    from opencompass.configs.datasets.SuperGLUE_ReCoRD.SuperGLUE_ReCoRD_gen_30dea0 import ReCoRD_datasets
    from opencompass.configs.datasets.SuperGLUE_WiC.SuperGLUE_WiC_gen_d06864 import WiC_datasets
    from opencompass.configs.datasets.SuperGLUE_WSC.SuperGLUE_WSC_gen_7902a7 import WSC_datasets
    from opencompass.configs.datasets.race.race_gen_69ee4f import race_datasets
    from opencompass.configs.datasets.Xsum.Xsum_gen_31397e import Xsum_datasets
    from opencompass.configs.datasets.gsm8k.gsm8k_gen_1d7fe4 import gsm8k_datasets
    from opencompass.configs.datasets.summedits.summedits_gen_315438 import summedits_datasets
    from opencompass.configs.datasets.math.math_gen_265cce import math_datasets
    from opencompass.configs.datasets.hellaswag.hellaswag_gen_6faab5 import hellaswag_datasets
    from opencompass.configs.datasets.ARC_e.ARC_e_gen_1e0de5 import ARC_e_datasets
    from opencompass.configs.datasets.ARC_c.ARC_c_gen_1e0de5 import ARC_c_datasets
    from opencompass.configs.datasets.piqa.piqa_gen_1194eb import piqa_datasets
    from opencompass.configs.datasets.siqa.siqa_gen_e78df3 import siqa_datasets
    from opencompass.configs.datasets.strategyqa.strategyqa_gen_1180a7 import strategyqa_datasets
    from opencompass.configs.datasets.winogrande.winogrande_gen_458220 import winogrande_datasets
    from opencompass.configs.datasets.obqa.obqa_gen_9069e4 import obqa_datasets
    from opencompass.configs.datasets.nq.nq_gen_c788f6 import nq_datasets
    from opencompass.configs.datasets.triviaqa.triviaqa_gen_2121ce import triviaqa_datasets
    from opencompass.configs.datasets.cmb.cmb_gen_dfb5c4 import cmb_datasets
    from opencompass.configs.datasets.cmmlu.cmmlu_gen_c13365 import cmmlu_datasets
    from opencompass.configs.datasets.PMMEval.pmmeval_gen import PMMEval_datasets

    # Note: to be supported
    # from opencompass.configs.datasets.flores.flores_gen_806ede import flores_datasets
    # from opencompass.configs.datasets.TheoremQA.TheoremQA_5shot_gen_6f0af8 import TheoremQA_datasets
    # from opencompass.configs.datasets.commonsenseqa.commonsenseqa_gen_c946f2 import commonsenseqa_datasets
    # from opencompass.configs.datasets.bbh.bbh_gen_5b92b0 import bbh_datasets


datasets = []
_locals = {k: v for k, v in locals().items() if k.endswith('_datasets')}

for k, v in _locals.items():
    for _dataset in v:
        _dataset['dataset_name'] = k.replace('_datasets', '')
        datasets.append(_dataset)


if __name__ == '__main__':
    for _dataset in datasets:
        print(_dataset)



# Copyright (c) Alibaba, Inc. and its affiliates.

import subprocess

if __name__ == '__main__':
    cmd = f'TEST_LEVEL_LIST=0,1 python3 -m unittest discover tests'
    run_res = subprocess.run(cmd, text=True, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    if run_res.returncode == 0:
        print(f'>>test_run_all stdout: {run_res.stdout}')
    else:
        print(f'>>test_run_all stderr: {run_res.stderr}')



from mteb import AbsTaskRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class T2Retrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="T2Retrieval",
        description="T2Ranking: A large-scale Chinese Benchmark for Passage Ranking",
        reference="https://arxiv.org/abs/2304.03679",
        dataset={
            "path": "C-MTEB/T2Retrieval",
            "revision": "8731a845f1bf500a4f111cf1070785c793d10e64",
            "qrel_revision": "1c83b8d1544e529875e3f6930f3a1fcf749a8e97",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{xie2023t2ranking,
      title={T2Ranking: A large-scale Chinese Benchmark for Passage Ranking}, 
      author={Xiaohui Xie and Qian Dong and Bingning Wang and Feiyang Lv and Ting Yao and Weinan Gan and Zhijing Wu and Xiangsheng Li and Haitao Li and Yiqun Liu and Jin Ma},
      year={2023},
      eprint={2304.03679},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 874.1184182791619,
                    "average_query_length": 10.938847974750132,
                    "num_documents": 118605,
                    "num_queries": 22812,
                    "average_relevant_docs_per_query": 5.213571804313519,
                }
            },
        },
    )


class MMarcoRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MMarcoRetrieval",
        description="MMarcoRetrieval",
        reference="https://arxiv.org/abs/2309.07597",
        dataset={
            "path": "C-MTEB/MMarcoRetrieval",
            "revision": "539bbde593d947e2a124ba72651aafc09eb33fc2",
            "qrel_revision": "bae08bb7bddbedb96c7e7db52018a55167b67f89",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{xiao2024cpack,
      title={C-Pack: Packaged Resources To Advance General Chinese Embedding}, 
      author={Shitao Xiao and Zheng Liu and Peitian Zhang and Niklas Muennighoff and Defu Lian and Jian-Yun Nie},
      year={2024},
      eprint={2309.07597},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 114.41787048392986,
                    "average_query_length": 10.51131805157593,
                    "num_documents": 106813,
                    "num_queries": 6980,
                    "average_relevant_docs_per_query": 1.0654727793696275,
                }
            },
        },
    )


class DuRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="DuRetrieval",
        description="A Large-scale Chinese Benchmark for Passage Retrieval from Web Search Engine",
        reference="https://aclanthology.org/2022.emnlp-main.357.pdf",
        dataset={
            "path": "C-MTEB/DuRetrieval",
            "revision": "a1a333e290fe30b10f3f56498e3a0d911a693ced",
            "qrel_revision": "497b7bd1bbb25cb3757ff34d95a8be50a3de2279",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{qiu2022dureaderretrieval,
      title={DuReader_retrieval: A Large-scale Chinese Benchmark for Passage Retrieval from Web Search Engine}, 
      author={Yifu Qiu and Hongyu Li and Yingqi Qu and Ying Chen and Qiaoqiao She and Jing Liu and Hua Wu and Haifeng Wang},
      year={2022},
      eprint={2203.10232},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 331.3219967800322,
                    "average_query_length": 9.289,
                    "num_documents": 100001,
                    "num_queries": 2000,
                    "average_relevant_docs_per_query": 4.9195,
                }
            },
        },
    )


class CovidRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CovidRetrieval",
        description="COVID-19 news articles",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/CovidRetrieval",
            "revision": "1271c7809071a13532e05f25fb53511ffce77117",
            "qrel_revision": "a9f41b7cdf24785531d12417ce0d1157ed4b39ca",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 332.4152658473415,
                    "average_query_length": 25.9304531085353,
                    "num_documents": 100001,
                    "num_queries": 949,
                    "average_relevant_docs_per_query": 1.0105374077976819,
                }
            },
        },
    )


class CmedqaRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CmedqaRetrieval",
        description="Online medical consultation text. Used the CMedQAv2 as its underlying dataset.",
        reference="https://aclanthology.org/2022.emnlp-main.357.pdf",
        dataset={
            "path": "C-MTEB/CmedqaRetrieval",
            "revision": "cd540c506dae1cf9e9a59c3e06f42030d54e7301",
            "qrel_revision": "279d737f36c731c8ff6e2b055f31fe02216fa23d",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 307.7710222897771,
                    "average_query_length": 48.470367591897976,
                    "num_documents": 100001,
                    "num_queries": 3999,
                    "average_relevant_docs_per_query": 1.86271567891973,
                }
            },
        },
    )


class EcomRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="EcomRetrieval",
        description="EcomRetrieval",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/EcomRetrieval",
            "revision": "687de13dc7294d6fd9be10c6945f9e8fec8166b9",
            "qrel_revision": "39c90699b034ec22ac45b3abf5b0bbb5ffd421f9",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 32.98041664189015,
                    "average_query_length": 6.798,
                    "num_documents": 100902,
                    "num_queries": 1000,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )


class MedicalRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MedicalRetrieval",
        description="MedicalRetrieval",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/MedicalRetrieval",
            "revision": "2039188fb5800a9803ba5048df7b76e6fb151fc6",
            "qrel_revision": "37b8efec53c54c3d9c6af212f6710b62ccdf895c",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 122.04231725066585,
                    "average_query_length": 17.938,
                    "num_documents": 100999,
                    "num_queries": 1000,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )


class VideoRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="VideoRetrieval",
        description="VideoRetrieval",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/VideoRetrieval",
            "revision": "58c2597a5943a2ba48f4668c3b90d796283c5639",
            "qrel_revision": "faa71382b6a29cf1778d1f436b963e75cb5b927c",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "dev": {
                    "average_document_length": 31.048855642524522,
                    "average_query_length": 7.365,
                    "num_documents": 100930,
                    "num_queries": 1000,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from typing import Any, Dict, Iterator, List

import json
from transformers import AutoTokenizer

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.api.base import ApiPluginBase
from evalscope.perf.plugin.registry import register_api
from evalscope.utils.logger import get_logger

logger = get_logger()


@register_api('custom')
class CustomPlugin(ApiPluginBase):
    """Support tensorrt-llm triton server
    """

    def __init__(self, mode_path: str):
        """Init the plugin

        Args:
            mode_path (str): The model path, we use the tokenizer
                weight in the model to calculate the number of the
                input and output tokens.
        """
        super().__init__(model_path=mode_path)
        if mode_path is not None:
            self.tokenizer = AutoTokenizer.from_pretrained(mode_path)
        else:
            self.tokenizer = None

    def build_request(self, messages: List[Dict], param: Arguments) -> Dict:
        """Build the openai format request based on prompt, dataset

        Args:
            message (Dict): The basic message to generator query.
            param (Arguments): The query parameters.

        Raises:
            Exception: NotImplemented

        Returns:
            Dict: The request body. None if prompt format is error.
        """
        try:
            query = json.loads(param.query_template)
            ApiPluginBase.replace_values(query, param.model, messages[0]['content'])
            return query
        except Exception as e:
            logger.exception(e)
            logger.error('Prompt: %s invalidate!' % messages)
            return None

    def parse_responses(self, responses, request: Any = None, **kwargs) -> Dict:
        """Parser responses and return number of request and response tokens.
           sample of the output delta:
           {"id":"4","object":"chat.completion.chunk","created":1714030870,"model":"llama3","choices":[{"index":0,"delta":{"role":"assistant","content":""},"logprobs":null,"finish_reason":null}]}


        Args:
            responses (List[bytes]): List of http response body, for stream output,
                there are multiple responses, for general only one.
            kwargs: (Any): The command line --parameter content.
        Returns:
            Tuple: Return number of prompt token and number of completion tokens.
        """
        full_response_content = ''
        delta_contents = {}
        input_tokens = None
        output_tokens = None
        for response in responses:
            data = json.loads(response)
            # {"context_logits":0.0,"cum_log_probs":0.0,"generation_logits":0.0,"model_name":"ensemble",
            # "model_version":"1","output_log_probs":[0.0,0.0,0.0,0.0,0.0],"sequence_end":false,"sequence_id":0,"sequence_start":false,"text_output":"性"}
            if 'text_output' in data:
                if 0 in delta_contents:
                    delta_contents[0].append(data['text_output'])
                else:
                    delta_contents[0] = [data['text_output']]
        if input_tokens is None and output_tokens is None and self.tokenizer is not None:
            input_tokens = 0
            output_tokens = 0
            for _, choice_contents in delta_contents.items():
                full_response_content = ''.join([m for m in choice_contents])
                input_tokens += len(self.tokenizer.encode(request['text_input']))
                output_tokens += len(self.tokenizer.encode(full_response_content))
        elif input_tokens is None and output_tokens is None:  # no usage info get.
            input_tokens = 0
            output_tokens = 0
            logger.warning('No usage info get.')

        return input_tokens, output_tokens



import os
import mteb
from evalscope.backend.rag_eval import EmbeddingModel
from evalscope.backend.rag_eval import cmteb
from mteb.task_selection import results_to_dataframe
from evalscope.utils.logger import get_logger

logger = get_logger()


def show_results(output_folder, model, results):
    model_name = model.mteb_model_meta.model_name_as_path()
    revision = model.mteb_model_meta.revision

    results_df = results_to_dataframe({model_name: {revision: results}})

    save_path = os.path.join(
        output_folder,
        model_name,
        revision,
    )
    logger.info(f"Evaluation results:\n{results_df.to_markdown()}")
    logger.info(f"Evaluation results saved in {os.path.abspath(save_path)}")


def one_stage_eval(
    model_args,
    eval_args,
) -> None:
    # load model
    model = EmbeddingModel.load(**model_args)
    custom_dataset_path = eval_args.pop("dataset_path", None)
    # load task first to update instructions
    tasks = cmteb.TaskBase.get_tasks(
        task_names=eval_args["tasks"], dataset_path=custom_dataset_path
    )
    evaluation = mteb.MTEB(tasks=tasks)

    # run evaluation
    results = evaluation.run(model, **eval_args)

    # save and log results
    show_results(eval_args["output_folder"], model, results)


def two_stage_eval(
    model1_args,
    model2_args,
    eval_args,
) -> None:
    """a two-stage run with the second stage reading results saved from the first stage."""
    # load model
    dual_encoder = EmbeddingModel.load(**model1_args)
    cross_encoder = EmbeddingModel.load(**model2_args)

    first_stage_path = f"{eval_args['output_folder']}/stage1"
    second_stage_path = f"{eval_args['output_folder']}/stage2"

    tasks = cmteb.TaskBase.get_tasks(task_names=eval_args["tasks"])
    for task in tasks:
        evaluation = mteb.MTEB(tasks=[task])

        # stage 1: run dual encoder
        evaluation.run(
            dual_encoder,
            save_predictions=True,
            output_folder=first_stage_path,
            overwrite_results=True,
            hub=eval_args["hub"],
            limits=eval_args["limits"],
        )
        # stage 2: run cross encoder
        results = evaluation.run(
            cross_encoder,
            top_k=eval_args["top_k"],
            save_predictions=True,
            output_folder=second_stage_path,
            previous_results=f"{first_stage_path}/{task.metadata.name}_default_predictions.json",
            overwrite_results=True,
            hub=eval_args["hub"],
            limits=eval_args["limits"],
        )

        # save and log results
        show_results(second_stage_path, cross_encoder, results)



# Copyright (c) Alibaba, Inc. and its affiliates.
import os
import shutil
import subprocess
import time
import unittest

import json
import requests

from evalscope.backend.vlm_eval_kit import VLMEvalKitBackendManager
from evalscope.run import run_task
from evalscope.summarizer import Summarizer
from evalscope.utils import is_module_installed, test_level_list
from evalscope.utils.logger import get_logger

logger = get_logger(__name__)

DEFAULT_CHAT_MODEL_URL = 'http://127.0.0.1:8000/v1/chat/completions'
DEFAULT_JUDGE_MODEL_URL = 'http://127.0.0.1:8866/v1/chat/completions'

DEFAULT_API_KEY = 'EMPTY'
DEFAULT_MODEL_NAME = 'CustomAPIModel'
DEFAULT_WORK_DIR = 'outputs'


class TestRunSwiftVLMEval(unittest.TestCase):

    def setUp(self) -> None:
        logger.info('Init env for swift-eval UTs ...\n')
        assert is_module_installed('llmuses'), 'Please install `llmuses` from pypi or source code.'

        logger.warning('Note: installing ms-vlmeval ...')
        subprocess.run('pip3 install ms-vlmeval -U', shell=True, check=True)

        logger.warning('Note: installing ms-swift ...')
        subprocess.run('pip3 install ms-swift -U', shell=True, check=True)

        if os.path.exists(DEFAULT_WORK_DIR):
            shutil.rmtree(DEFAULT_WORK_DIR)
            logger.info(f'Removed work dir: {os.path.abspath(DEFAULT_WORK_DIR)} \n')

        logger.info('\nStaring run swift deploy ...')
        self.model_name = 'qwen-vl-chat'
        self.process_swift_deploy = subprocess.Popen(
            f'swift deploy --model_type {self.model_name}',
            text=True,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        logger.info('\nStaring run swift deploy judge ...')
        self.judge_model_name = 'qwen2-7b-instruct'
        self.process_swift_deploy_judge = subprocess.Popen(
            f'swift deploy --model_type {self.judge_model_name} \
                                                    --port 8866',
            text=True,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        self.all_datasets = VLMEvalKitBackendManager.list_supported_datasets()
        assert len(self.all_datasets) > 0, f'Failed to list datasets from VLMEvalKit backend: {self.all_datasets}'

    def tearDown(self) -> None:
        # Stop the swift deploy model service
        logger.warning('\nStopping swift deploy ...')
        self.process_swift_deploy.terminate()
        self.process_swift_deploy.wait()

        self.process_swift_deploy_judge.terminate()
        self.process_swift_deploy_judge.wait()

        logger.info('Process swift-deploy terminated successfully.')

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f'{module_name} is installed.')
        else:
            raise ModuleNotFoundError(f'run: pip install {module_name}')

    @staticmethod
    def check_service_status(url: str, data: dict, retries: int = 20, delay: int = 10):
        for i in range(retries):
            try:
                logger.info(f'Attempt {i + 1}: Checking service at {url} ...')
                response = requests.post(
                    url, data=json.dumps(data), headers={'Content-Type': 'application/json'}, timeout=30)
                if response.status_code == 200:
                    logger.info(f'Service at {url} is available !\n\n')
                    return True
                else:
                    logger.info(f'Service at {url} returned status code {response.status_code}.')
            except requests.exceptions.RequestException as e:
                logger.info(f'Attempt {i + 1}: An error occurred: {e}')

            time.sleep(delay)

        logger.info(f'Service at {url} is not available after {retries} retries.')
        return False

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_api_with_judge(self):
        # Check the judge service status
        data = {'model': self.judge_model_name, 'messages': [{'role': 'user', 'content': 'who are you?'}]}
        assert self.check_service_status(DEFAULT_JUDGE_MODEL_URL, data=data), \
            f'Failed to check judge service status: {DEFAULT_JUDGE_MODEL_URL}'

        api_base = DEFAULT_CHAT_MODEL_URL
        task_cfg = {
            'eval_backend': 'VLMEvalKit',
            'eval_config': {
                'LOCAL_LLM':
                self.judge_model_name,  # judge model id
                'OPENAI_API_BASE':
                DEFAULT_JUDGE_MODEL_URL,  # judge model api
                'OPENAI_API_KEY':
                DEFAULT_API_KEY,
                'data': ['SEEDBench_IMG', 'ChartQA_TEST'],
                'limit':
                20,
                'mode':
                'all',
                'model': [{
                    'api_base': api_base,  # swfit deploy model api
                    'key': DEFAULT_API_KEY,
                    'name': DEFAULT_MODEL_NAME,  # must be CustomAPIModel for swift
                    'temperature': 0.0,
                    'type': self.model_name
                }],  # swift model type
                'nproc':
                1,
                'reuse':
                True,
                'work_dir':
                DEFAULT_WORK_DIR
            }
        }

        # Check the service status
        data = {'model': self.model_name, 'messages': [{'role': 'user', 'content': 'who are you?'}]}
        assert self.check_service_status(api_base, data=data), f'Failed to check service status: {api_base}'

        logger.info(f'>> Start to run task: {task_cfg}')

        run_task(task_cfg)

        logger.info('>> Start to get the report with summarizer ...')
        report_list = Summarizer.get_report_from_cfg(task_cfg)
        logger.info(f'\n>> The report list: {report_list}')

        assert len(report_list) > 0, f'Failed to get report list: {report_list}'


if __name__ == '__main__':
    unittest.main(buffer=False)



# Copyright (c) Alibaba, Inc. and its affiliates.
from typing import Union

from evalscope.utils import yaml_to_dict


class BackendManager:
    def __init__(self, config: Union[str, dict], **kwargs):
        """
        BackendManager is the base class for the evaluation backend manager.
        It provides the basic configuration parsing, command generation, task submission, and result fetching.

        config: str or dict, the configuration of the evaluation backend.
            could be a string of the path to the configuration file (yaml), or a dictionary.
        """
        if isinstance(config, str):
            self.config_d = yaml_to_dict(config)
        else:
            self.config_d = config

        self.kwargs = kwargs

    def run(self, *args, **kwargs):
        """
        Run the evaluation backend.
        """
        raise NotImplementedError



import os
import torch
import torch.nn.functional as F
from typing import List
from PIL import Image
from evalscope.backend.rag_eval.utils.tools import download_model, PIL_to_base64
from transformers import AutoModel, AutoProcessor
from langchain_core.embeddings import Embeddings


class VisionModel:
    @staticmethod
    def load(**kw):
        api_base = kw.get("api_base", None)
        if api_base:

            return VLMAPI(
                model_name=kw.get("model_name", ""),
                openai_api_base=api_base,
                openai_api_key=kw.get("api_key", "EMPTY"),
                prompt=kw.get("prompt", None),
            )
        else:
            return CLIPModel(**kw)


class VLMAPI:
    def __init__(self, model_name, openai_api_base, openai_api_key, prompt=None):
        from langchain_openai import ChatOpenAI
        from langchain_core.prompts import ChatPromptTemplate

        self.model_name = model_name
        self.model = ChatOpenAI(
            model_name=model_name,
            openai_api_base=openai_api_base,
            openai_api_key=openai_api_key,
        )
        self.default_prompt = "Please describe this image in general. Directly provide the description, do not include prefix like 'This image depicts'"
        self.prompt = ChatPromptTemplate.from_messages(
            [
                ("system", prompt if prompt else self.default_prompt),
                (
                    "user",
                    [
                        {
                            "type": "image_url",
                            "image_url": {"url": "data:image/jpeg;base64,{image_data}"},
                        }
                    ],
                ),
            ]
        )
        self.chain = self.prompt | self.model
        self.transform = PIL_to_base64

    def encode_image(self, images):
        captions = []
        for image in images:
            response = self.chain.invoke({"image_data": image})
            captions.append(response.content)
        return captions


class CLIPModel(Embeddings):
    def __init__(
        self,
        model_name: str,
        revision: str = "master",
        hub="modelscope",
        device="cpu",
    ):
        self.device = device
        self.model_name = model_name
        self.revision = revision

        # Download the model if it doesn't exist locally
        if not os.path.exists(model_name) and hub == "modelscope":
            model_name = download_model(self.model_name, self.revision)

        # Load the model and processor
        self.model = AutoModel.from_pretrained(model_name).to(self.device)
        self.processor = AutoProcessor.from_pretrained(model_name)
        self.transform = self.processor.image_processor
        self.tokenizer = self.processor.tokenizer

    def encode_text(self, batch_texts: List[str] | List[List[str]]):
        if isinstance(batch_texts[0], list):
            batch_texts = [
                text for _, texts in enumerate(batch_texts) for text in texts
            ]
        # Ensure that the input texts are within the token limit
        max_length = self.tokenizer.model_max_length
        if not max_length or max_length > 0xFFFFFF:
            max_length = 512
        encoded_inputs = self.tokenizer(
            text=batch_texts,
            max_length=max_length,
            padding=True,
            truncation=True,
            return_tensors="pt",
        )

        inputs = {k: v.to(self.device) for k, v in encoded_inputs.items()}

        with torch.no_grad():
            text_features = self.model.get_text_features(**inputs)
        text_features = F.normalize(text_features, p=2, dim=-1)
        return text_features

    def encode_image(self, image):
        batch_images = torch.stack([d["pixel_values"][0] for d in image])
        batch_images = batch_images.to(self.device)
        with torch.no_grad():
            image_features = self.model.get_image_features(batch_images)
        image_features = F.normalize(image_features, p=2, dim=-1)
        return image_features

    def embed_documents(self, texts):
        text_features = self.encode_text(texts)
        return text_features.cpu().numpy().tolist()

    def embed_query(self, text):
        text_features = self.encode_text([text])
        return text_features.cpu().numpy().tolist()[0]

    def embed_image(self, uris: List[str]):
        # read image and transform
        images = [Image.open(image_path) for image_path in uris]
        transformed_images = [
            self.transform(
                image,
                return_tensors="pt",
            )
            for image in images
        ]
        image_features = self.encode_image(transformed_images)
        return image_features.cpu().numpy().tolist()


if __name__ == "__main__":
    model = CLIPModel("AI-ModelScope/chinese-clip-vit-large-patch14-336px")
    model.embed_image(
        [
            "custom_eval/multimodal/images/AMNH.jpg",
            "custom_eval/multimodal/images/AMNH.jpg",
        ]
    )
    model.encode_text(["我喜欢吃饭" * 1000])
    print("done")



# Copyright (c) Alibaba, Inc. and its affiliates.

import subprocess
import unittest
from evalscope.utils import test_level_list, is_module_installed
from evalscope.utils.logger import get_logger
from evalscope.run import run_task

logger = get_logger()


class TestMTEB(unittest.TestCase):

    def setUp(self) -> None:
        self._check_env('mteb')

    def tearDown(self) -> None:
        pass

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f'{module_name} is installed.')
        else:
            raise ModuleNotFoundError(f'run: pip install {module_name}')

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_one_stage_mteb(self):
        task_cfg = {
            'eval_backend': 'RAGEval',
            'eval_config': {
                'tool': 'MTEB',
                'model': [
                    {
                        'model_name_or_path': 'AI-ModelScope/m3e-base',
                        'pooling_mode': None,  # load from model config
                        'max_seq_length': 512,
                        'prompt': '',
                        'model_kwargs': {'torch_dtype': 'auto'},
                        'encode_kwargs': {
                            'batch_size': 128,
                        },
                    }
                ],
                'eval': {
                    'tasks': [
                        'TNews',
                        'CLSClusteringS2S',
                        'T2Reranking',
                        'T2Retrieval',
                        'ATEC',
                    ],
                    'verbosity': 2,
                    'output_folder': 'outputs',
                    'overwrite_results': True,
                    'limits': 500,
                },
            },
        }

        run_task(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_two_stage_mteb(self):
        task_cfg = {
            'eval_backend': 'RAGEval',
            'eval_config': {
                'tool': 'MTEB',
                'model': [
                    {
                        'model_name_or_path': 'AI-ModelScope/m3e-base',
                        'is_cross_encoder': False,
                        'max_seq_length': 512,
                        'prompt': '',
                        'model_kwargs': {'torch_dtype': 'auto'},
                        'encode_kwargs': {
                            'batch_size': 64,
                        },
                    },
                    {
                        'model_name_or_path': 'OpenBMB/MiniCPM-Reranker',
                        'is_cross_encoder': True,
                        'max_seq_length': 512,
                        'prompt': '为这个问题生成一个检索用的表示',
                        'model_kwargs': {'torch_dtype': 'auto'},
                        'encode_kwargs': {
                            'batch_size': 32,
                        },
                    },
                ],
                'eval': {
                    'tasks': ['MedicalRetrieval', 'T2Retrieval'],
                    'verbosity': 2,
                    'output_folder': 'outputs',
                    'overwrite_results': True,
                    'limits': 10,
                },
            },
        }

        run_task(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_custom(self):
        task_cfg = {
            'eval_backend': 'RAGEval',
            'eval_config': {
                'tool': 'MTEB',
                'model': [
                    {
                        'model_name_or_path': 'AI-ModelScope/m3e-base',
                        'pooling_mode': None,  # load from model config
                        'max_seq_length': 512,
                        'prompt': '',
                        'model_kwargs': {'torch_dtype': 'auto'},
                        'encode_kwargs': {
                            'batch_size': 128,
                        },
                    }
                ],
                'eval': {
                    'tasks': ['CustomRetrieval'],
                    'dataset_path': 'custom_eval/text/retrieval',
                    'verbosity': 2,
                    'output_folder': 'outputs',
                    'overwrite_results': True,
                    'limits': 500,
                },
            },
        }

        run_task(task_cfg)


if __name__ == '__main__':
    unittest.main(buffer=False)



# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.mmlu.mmlu_adapter import DATASET_ID, SUBJECT_MAPPING, SUBSET_LIST, MMLUAdapter
from evalscope.benchmarks.mmlu.mmlu_adapter import MMLUAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa



import os
import torch
from torch.utils.data import DataLoader, Dataset as TorchDataset
from evalscope.utils.logger import get_logger


logger = get_logger()


def build_dataset(
    dataset_name,
    root=None,
    transform=None,
    split="test",
    wds_cache_dir=None,
    **kwargs,
):
    """
    Main function to use in order to build a dataset instance,

    dataset_name: str
        name of the dataset

    root: str
        root folder where the dataset is downloaded and stored. can be shared among datasets.

    transform: torchvision transform applied to images

    split: str
        split to use, depending on the dataset can have different options.
        In general, `train` and `test` are available.
        For specific splits, please look at the corresponding dataset.

    custom_classname_file: str or None
        Custom classname file where keys are dataset names and values are list of classnames.

    custom_template_file: str or None
        Custom template file where keys are dataset names and values are list of prompts, or dicts
        where keys are classnames and values are class-specific prompts.

    """

    if dataset_name == "dummy":
        ds = Dummy()
    elif dataset_name == "custom":
        ds = build_custom_dataset(dataset_name, data_dir=root, transform=transform)
    else:
        # WebDataset support using `webdataset` library
        ds = build_wds_dataset(
            dataset_name,
            transform=transform,
            split=split,
            data_dir=root,
            cache_dir=wds_cache_dir,
        )

    return ds


class Dummy:

    def __init__(self):
        self.classes = ["blank image", "noisy image"]

    def __getitem__(self, i):
        return torch.zeros(3, 224, 224), 0

    def __len__(self):
        return 1


class DatasetWrapper(TorchDataset):
    def __init__(self, dataset, transform=None, image_key="image", text_key="query"):
        self.dataset = dataset
        self.transform = transform
        self.image_key = image_key
        self.text_key = text_key

    def __len__(self):
        return len(self.dataset)

    def __getitem__(self, idx):
        item = self.dataset[idx]

        # 加载图像
        image = item[self.image_key]
        if self.transform is not None:
            image = self.transform(image, return_tensors="pt")

        # 获取查询列表
        query = item[self.text_key]
        if isinstance(query, str):
            query = [query]

        return image, query


def get_dataset_default_task(dataset):
    if dataset in (
        "custom",
        "muge",
        "flickr30k",
        "flickr8k",
        "mscoco_captions",
        "mscoco_captions2017",
        "multilingual_mscoco_captions",
        "flickr30k-200",
        "crossmodal3600",
        "xtd200",
    ):
        return "zeroshot_retrieval"
    else:
        return "zeroshot_classification"


def get_dataloader(dataset_name, dataset, batch_size, num_workers):
    if dataset_name == "custom":
        dataloader = DataLoader(
            dataset,
            batch_size=batch_size,
            shuffle=False,
            num_workers=num_workers,
            collate_fn=image_captions_collate_fn,
        )
    else:
        dataloader = DataLoader(
            dataset.batched(batch_size),
            batch_size=None,
            shuffle=False,
            num_workers=num_workers,
        )
    return dataloader


def image_captions_collate_fn(batch):
    transposed = list(zip(*batch))
    imgs = transposed[0]
    texts = transposed[1]
    return imgs, texts


def build_custom_dataset(dataset_name, data_dir, transform=None):
    from datasets import load_dataset, Features, Image, Sequence, Value

    qrels_ds = load_dataset(
        "json",
        data_files=os.path.join(data_dir, "image_queries.jsonl"),
        features=Features(
            {"image_path": Image(decode=True), "query": Sequence(Value("string"))}
        ),
        split="train",
    )

    dataset = DatasetWrapper(
        qrels_ds, transform, image_key="image_path", text_key="query"
    )
    return dataset


def build_wds_dataset(
    dataset_name, transform, split="test", data_dir="root", cache_dir=None
):
    """
    Load a dataset in WebDataset format. Either local paths or HTTP URLs can be specified.
    Expected file structure is:
    ```
    data_dir/
        train/
            nshards.txt
            0.tar
            1.tar
            ...
        test/
            nshards.txt
            0.tar
            1.tar
            ...
        classnames.txt
        zeroshot_classification_templates.txt
        dataset_type.txt
    ```
    Classnames and templates are required for zeroshot classification, while dataset type
    (equal to "retrieval") is required for zeroshot retrieval datasets.

    You can use the `clip_benchmark_export_wds` or corresponding API
    (`clip_benchmark.webdataset_builder.convert_dataset`) to convert datasets to this format.

    Set `cache_dir` to a path to cache the dataset, otherwise, no caching will occur.
    """
    import webdataset as wds

    def read_txt(fname):
        if "://" in fname:
            stream = os.popen("curl -L -s --fail '%s'" % fname, "r")
            value = stream.read()
            if stream.close():
                raise FileNotFoundError("Failed to retreive data")
        else:
            with open(fname, "r") as file:
                value = file.read()
        return value

    if not data_dir:
        data_dir = f"https://modelscope.cn/datasets/clip-benchmark/wds_{dataset_name}/resolve/master"

    # Git LFS files have a different file path to access the raw data than other files
    if data_dir.startswith("https://modelscope.cn/datasets"):
        *split_url_head, _, url_path = data_dir.split("/", 7)
        url_head = "/".join(split_url_head)
        metadata_dir = "/".join([url_head, "resolve", url_path])
        tardata_dir = "/".join([url_head, "resolve", url_path])
    else:
        metadata_dir = tardata_dir = data_dir
    # Get number of shards
    nshards_fname = os.path.join(metadata_dir, split, "nshards.txt")
    nshards = int(
        read_txt(nshards_fname)
    )  # Do not catch FileNotFound, nshards.txt should be mandatory
    
    # Get dataset type (classification or retrieval)
    type_fname = os.path.join(metadata_dir, "dataset_type.txt")
    try:
        dataset_type = read_txt(type_fname).strip().lower()
    except FileNotFoundError:
        dataset_type = "classification"
    
    filepattern = os.path.join(tardata_dir, split, "{0..%d}.tar" % (nshards - 1))
    # Load webdataset (support WEBP, PNG, and JPG for now)
    if not cache_dir or not isinstance(cache_dir, str):
        cache_dir = None
    else:
        os.makedirs(cache_dir, exist_ok=True)
    dataset = wds.WebDataset(
        filepattern,
        cache_dir=cache_dir,
        nodesplitter=lambda src: src,
        shardshuffle=False,
        verbose=True,
    ).decode(
        wds.autodecode.ImageHandler("pil", extensions=["webp", "png", "jpg", "jpeg"])
    )
    
    # Load based on classification or retrieval task
    if dataset_type == "retrieval":
        dataset = dataset.to_tuple(["webp", "png", "jpg", "jpeg"], "txt").map_tuple(
            transform, str.splitlines
        )
        dataset.classes = dataset.templates = None
    else:
        label_type = (
            "npy" if dataset_type == "multilabel" else "cls"
        )  # Special case for multilabel
        dataset = dataset.to_tuple(
            ["webp", "png", "jpg", "jpeg"], label_type
        ).map_tuple(transform, None)
        # Get class names if present
        classnames_fname = os.path.join(metadata_dir, "classnames.txt")
        try:
            dataset.classes = [
                line.strip() for line in read_txt(classnames_fname).splitlines()
            ]
        except FileNotFoundError:
            logger.warning("WARNING: classnames.txt not found")
            dataset.classes = None
        # Get zeroshot classification templates if present
        templates_fname = os.path.join(
            metadata_dir, "zeroshot_classification_templates.txt"
        )
        try:
            dataset.templates = [
                line.strip() for line in read_txt(templates_fname).splitlines()
            ]
        except FileNotFoundError:
            logger.warning("WARNING: zeroshot_classification_templates.txt not found")
            dataset.templates = None

    return dataset
